unit Fonts;

    (*********************************************************************
     *  unit Fonts                                                       *
     *                                                                   *
     *  home page : http://www.dsplayer.de                               *
     *  email     : coder@dsplayer.de                                    *
     *                                                                   *
     *                                                                   *
     *  date      : 15-03-2004                                           *
     *                                                                   *
     *  The contents of this file are used with permission, subject to   *
     *  the Mozilla Public License Version 1.1 (the "License"); you may  *
     *  not use this file except in compliance with the License. You may *
     *  obtain a copy of the License at                                  *
     *  http://www.mozilla.org/MPL/MPL-1.1.html                          *
     *                                                                   *
     *  Software distributed under the License is distributed on an      *
     *  "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   *
     *  implied. See the License for the specific language governing     *
     *  rights and limitations under the License.                        *
     *                                                                   *
     *********************************************************************)

{
  @abstract(DSPlayer Components.)
  @author(Martin Offenwanger: coder@dsplayer.de)
  @created(Mar 15, 2004)
  @lastmod(Apr 19, 2004)
}

interface

uses types, SysUtils, Graphics, windows, messages;


////////////////////////////////////////////////////////////////////////////////
// class TFonts Declaration
// OS: Windows
// Filetypes: TTF only !
//------------------------------------------------------------------------------
//This is TTF file header
type
  TT_OFFSET_TABLE = record
    uMajorVersion       : Word;
    uMinorVersion       : Word;
    uNumOfTables        : Word;
    uSearchRange        : Word;
    uEntrySelector      : Word;
    uRangeShift         : Word;
end;
// Tables in TTF file and there placement and name (tag)
type
  TT_TABLE_DIRECTORY = record
    szTag       : string[3];    // table name
    uCheckSum   : DWord;        // Check sum
    uOffset     : DWord;        // Offset from beginning of file
    uLength     : DWord;        // length of the table in bytes}
end;
// Header of names table
type
  TT_NAME_TABLE_HEADER = record
    uFSelector          : Word; // format selector. Always 0
    uNRCount            : Word; // Name Records count
    uStorageOffset      : Word; // Offset for strings storage,
                                // from start of the table
end;
// Record in names table
type
  TT_NAME_RECORD = record
    uPlatformID         : Word;
    uEncodingID         : Word;
    uLanguageID         : Word;
    uNameID             : Word;
    uStringLength       : Word;
    uStringOffset       : Word; //from start of storage area
end;

type TFonts = class
private
  m_Font: TFont;
  m_Log_Font: LOGFONT;
  m_Font_File: string;
  m_Font_Name: string;
  function Swap32(Value: DWord): DWord;
public
  constructor Create(f_Font_File: string);
  destructor  Destroy();
  procedure   SetSize(f_Fontsize: Integer);
  procedure   SetColor(f_red: integer;f_green: integer;f_blue: Integer);
  procedure   SetFontStyles(f_Font_Styles: string);
  function    GetFont(): TFont;
  function    GetFontName: String;
  function    getTTFName(f_file: String): string; 
end;
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////


implementation

uses globals;


////////////////////////////////////////////////////////////////////////////////
// class TFonts Implementation
//------------------------------------------------------------------------------
function TFonts.GetFontName: String;
begin
 RESULT := m_Font_Name;
end;
//------------------------------------------------------------------------------
constructor TFonts.Create(f_Font_File: string);
var
  l_ErrStr: string;
  l_msg: TMSG;
begin
  m_Font_File := f_Font_File;
  m_Font_Name := getTTFName(m_Font_File);
        try
            if (AddFontResource(pchar(f_Font_File))=0) then begin
                l_ErrStr := 'Can not add Font Resource File';
                abort;
            end;

            if not PostMessage(HWND_BROADCAST,WM_FONTCHANGE,0,0) then begin
                l_ErrStr := 'Can not add Font Resource';
                abort;
            end;

            m_Font  := TFont.Create;
            if (StrCopy(m_Log_Font.lfFaceName, PChar(m_Font_Name))=nil) then begin
                l_ErrStr := 'Can not set LogFont FaceName';
                abort;
            end;
            m_Log_Font.lfCharSet := DEFAULT_CHARSET;
            m_Font.Handle := 0;
            m_Font.Handle := CreateFontIndirect(m_Log_Font);
            if (m_Font.Handle=0) then begin
                l_ErrStr := 'Can not set LogFont FaceName';
                abort;
            end;
        except
          m_Font_File := '';
          m_Font_Name := '';
          m_Font.Handle := 0;
          g_ErrLog.AddErrStr('TFonts.Create' + l_ErrStr);
        end;
end;
//------------------------------------------------------------------------------
destructor TFonts.Destroy();
var
  l_ErrStr: string;
  l_Error:    integer;
begin
m_Font.Destroy;
        try
             if not (RemoveFontResource(pchar(m_Font_File))) then begin
                l_ErrStr := 'Can not remove Remove Font Resource file';
                abort;
             end;
             if not PostMessage(HWND_BROADCAST,WM_FONTCHANGE,0,0) then begin
                l_ErrStr := 'Can not remove Remove Font Resource';
                abort;
             end;
        except
                g_ErrLog.AddErrStr('TFonts.Destroy' + l_ErrStr);
        end;
end;
//------------------------------------------------------------------------------
procedure TFonts.SetSize(f_Fontsize: Integer);
begin
  m_Font.Size := f_Fontsize;
end;
//------------------------------------------------------------------------------
procedure TFonts.SetColor(f_red: integer;f_green: integer;f_blue: Integer);
begin
 m_Font.Color := RGB(f_red,f_green,f_blue);
end;
//------------------------------------------------------------------------------
function TFonts.GetFont(): TFont;
begin
 RESULT := m_Font;
end;
//------------------------------------------------------------------------------
procedure TFonts.SetFontStyles(f_Font_Styles: string);
begin
  if( Pos( 'Bold', f_Font_Styles ) > 0 )then
        m_Font.Style := m_Font.Style  + [ fsBold ];
  if( Pos( 'Italic', f_Font_Styles ) > 0 )then
        m_Font.Style := m_Font.Style  + [ fsItalic ];
  if( Pos( 'Underline', f_Font_Styles ) > 0 )then
        m_Font.Style := m_Font.Style  + [ fsUnderline ];
  if( Pos( 'Strikeout', f_Font_Styles ) > 0 )then
      m_Font.Style := m_Font.Style  + [ fsStrikeout ];
end;
//------------------------------------------------------------------------------
function TFonts.getTTFName(f_file: String): string;
var
  ttOffSetTable : TT_OFFSET_TABLE;
  tblDir        : TT_TABLE_DIRECTORY;
  ttNTHeader    : TT_NAME_TABLE_HEADER;
  ttRecord      : TT_NAME_RECORD;
  ttfFile       : File;
  TTFNAme       : string[255];
  TTFNameResult : string;
  anzahl_bytes  : longint;
  FilePosition  : LongInt;
  bFound        : boolean;
  i,j           : integer;
  TempFileMode  : Byte;
begin                               
  RESULT := '';
  Assign(ttfFile,f_file);
  TempFileMode := FileMode;
  FileMode := fmOpenRead;
  reset(ttfFile,1);
  FileMode := TempFileMode;
  // read file header
  blockread(ttfFile,ttOffSetTable,sizeof(ttOffSetTable),anzahl_bytes);
  // rearrange bytes in the field
  ttOffSetTable.uNumOfTables  := Swap(ttOffSetTable.uNumOfTables);
  ttOffsetTable.uMajorVersion := Swap(ttOffsetTable.uMajorVersion);
  ttOffsetTable.uMinorVersion := Swap(ttOffsetTable.uMinorVersion);
  //check is this is a true type font and the version is 1.0
  if(ttOffSetTable.uMajorVersion <> 1) or (ttOffsetTable.uMinorVersion <> 0) then
        exit;
  bFound := false;
  for i := 0 to ttOffsetTable.uNumOfTables-1 do begin
        blockread(ttfFile,tblDir,sizeof(tblDir),anzahl_bytes);
        if uppercase(tblDir.szTag[0]+tblDir.szTag[1]+tblDir.szTag[2]+tblDir.szTag[3]) = 'NAME' then begin
                bFound := true;
                tblDir.uLength := Swap32(tblDir.uLength);
                tblDir.uOffset := Swap32(tblDir.uOffset);
                break;
        end;
  end;
  seek(ttfFile,0);
  if (bFound) then begin
        seek(ttfFile,tblDir.uOffset);
        blockread(ttfFile,ttNTHeader,sizeof(ttNTHeader),anzahl_bytes);
        ttNTHeader.uNRCount := swap(ttNTHeader.uNRCount);
        ttNTHeader.uStorageOffset := swap(ttNTHeader.uStorageOffset);
        for i := 0 to ttNTHeader.uNRCount-1 do begin
                blockread(ttfFile,ttRecord,sizeof(ttRecord),anzahl_bytes);
                ttRecord.uNameID := swap(ttRecord.uNameID);
                if ttRecord.uNameID = 1 then begin
                        ttRecord.uStringLength := Swap(ttRecord.uStringLength);
                        ttRecord.uStringOffset := Swap(ttRecord.uStringOffset);
                        FilePosition := FilePos(ttfFile);
                        seek(ttfFile,tblDir.uOffset + ttRecord.uStringOffset + ttNTHeader.uStorageOffset);
                        blockread(ttfFile,TTFName,ttRecord.uStringLength,anzahl_bytes);
                        if TTFNAme[0] = '' then
                                seek(ttfFile,FilePosition)
                        else begin
                                TTFNameResult := '';
                                for j := 0 to anzahl_bytes-1 do begin
                                        TTFNameResult := TTFNameResult + TTFName[j];
                                end;
                                RESULT := TTFNameResult;
                                close(ttfFile);
                                exit;
                       end;
                end;
        end;
  end;
close(ttfFile);
end;
//------------------------------------------------------------------------------
function TFonts.Swap32(Value: DWord): DWord;
begin
  Result := Swap(Value shr 16) or (Swap(Value) shl 16);
end;
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////



end.
