unit Debug;
{*******************************************************************************

Short Description :  DSPlayer LogDebug class
                     - supports Multithreading
Author            :  Martin Offenwanger
E-Mail            :  Coder@dsplayer.de
Homepage          :  www.dsplayer.de

     *  The contents of this file are used with permission, subject to   *
     *  the Mozilla Public License Version 1.1 (the "License"); you may  *
     *  not use this file except in compliance with the License. You may *
     *  obtain a copy of the License at                                  *
     *  http://www.mozilla.org/MPL/MPL-1.1.html                          *
     *                                                                   *
     *  Software distributed under the License is distributed on an      *
     *  "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   *
     *  implied. See the License for the specific language governing     *
     *  rights and limitations under the License.                        *

*******************************************************************************}
//------------------------------------------------------------------------------
interface
//------------------------------------------------------------------------------
uses WIndows, Dialogs,SysUtils,SyncObjs;
//------------------------------------------------------------------------------
type
  DspException = class(Exception);
////////////////////////////////////////////////////////////////////////////////
//  Class TErrLog Declaration
//------------------------------------------------------------------------------
type TErrLog = class
  private
        m_LogFile: string;
        m_EnableLogFile: boolean;
        m_ConsoleOutput: boolean;
        m_CriticalSection: TCriticalSection;
  public
        constructor Create(f_LogFile: string = ''; f_OpenConsole: boolean = false);
        destructor Destroy();
        procedure AddErrStr(f_ErrStr: string);
end;
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
implementation
////////////////////////////////////////////////////////////////////////////////
//  Class TErrLog Implementation
//------------------------------------------------------------------------------
constructor TErrLog.Create(f_LogFile: string;f_OpenConsole: boolean);
var l_File: TextFile;
begin
  m_CriticalSection := TCriticalSection.Create();
  if f_LogFile <> '' then begin
        m_EnableLogFile := true;
        try
                m_LogFile := f_LogFile;
                Assign(l_File, f_LogFile);
                Rewrite(l_File);
                CloseFile(l_File);
        except
                showmessage('TErrLog.Create , Creating Logfile failed');
        end;
  end else
        m_EnableLogFile := false;
  if (f_OpenConsole) then begin
       if (AllocConsole() = false)  then begin
               m_ConsoleOutput := false;
               showmessage('TErrLog.Create , Can not allocate Console');
       end else begin
                m_ConsoleOutput := true;
                writeln('Allocate Console Successfully');
       end;
  end else begin
        m_ConsoleOutput := false;
  end;
end;
//------------------------------------------------------------------------------
procedure TErrLog.AddErrStr(f_ErrStr: string);
var l_File: TextFile;
begin           
  m_CriticalSection.Enter;
  if (m_EnableLogFile) then begin
        try
                Assign(l_File, m_LogFile);
                Append(l_File);
                WriteLn(l_File, f_ErrStr);
                CloseFile(l_File);
        except
                showmessage('TErrLog.Create , Creating Logfile failed');
        end;
  end;
  if (m_ConsoleOutput) then begin
        writeln(f_ErrStr);
  end;
  m_CriticalSection.Leave;
end;
//------------------------------------------------------------------------------
destructor TErrLog.Destroy();
begin
  m_CriticalSection.Destroy();
end;
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
end.
