unit dspLanguage;

    (*********************************************************************
     *  unit dspLanguage                                                 *
     *                                                                   *
     *  home page : http://www.dsplayer.de                               *
     *  email     : coder@dsplayer.de                                    *
     *                                                                   *
     *                                                                   *
     *  date      : 15-03-2004                                           *
     *                                                                   *
     *  The contents of this file are used with permission, subject to   *
     *  the Mozilla Public License Version 1.1 (the "License"); you may  *
     *  not use this file except in compliance with the License. You may *
     *  obtain a copy of the License at                                  *
     *  http://www.mozilla.org/MPL/MPL-1.1.html                          *
     *                                                                   *
     *  Software distributed under the License is distributed on an      *
     *  "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   *
     *  implied. See the License for the specific language governing     *
     *  rights and limitations under the License.                        *
     *                                                                   *
     *********************************************************************)

{
  @abstract(DSPlayer Components.)
  @author(Martin Offenwanger: coder@dsplayer.de)
  @created(Mar 15, 2004)
  @lastmod(Apr 19, 2004)
}

interface

uses dspPopupDatatypes, Classes, SysUtils, IniFiles, FileCtrl,
     Forms;

type
  TdspLanguage = class
  public
    constructor Create();
    function setConfigSaveFile(f_configSaveFile: string): boolean;
    function setLanguageFileByDecription(f_fileDescription: string): boolean;
    function setLanguageFilePath(f_languageFilePath: string): boolean;
    function getLanguageDescriptions(out f_Descriptions: TStrings): boolean;
    function getPopupMenuCaptions(out f_PopupmenuData: TStrings;
                                      f_menuItemNames: TStrings): boolean;
    function getActiveLanguageAsDescription: string;
  private
    m_configSaveFile: String;
    m_languageFilePath: String;
    m_languageFile: String;
    m_currentLanguageDescription: String;
    function SaveDescriptionToConfgSaveFile(f_fileDescription: string): boolean;
end;

implementation

uses Globals;

const
  lc_iniPopupmenuSection        = 'popupmenus';
  lc_iniLangFileIdent           = 'language';
  lc_iniLangDescriptionIdent    = 'description';

constructor TdspLanguage.Create();
begin
    m_configSaveFile := '';
    m_languageFilePath := '';
    m_languageFile := '';
    m_currentLanguageDescription := '';
end;

function TdspLanguage.setConfigSaveFile(f_configSaveFile: string): boolean;
var
  l_ini: TInifile;
  l_temp: string;
begin
  Result := false;
  if not FileExists(f_configSaveFile) then begin
        g_errLog.AddErrStr('TdspLanguage.setConfigSaveFile: f_configSaveFile do not exist');
        exit;
  end;
  l_ini := TIniFile.Create(f_configSaveFile);
  try
        l_temp := l_ini.ReadString(lc_iniLangFileIdent,lc_iniLangDescriptionIdent,'failed');
        if l_temp = 'failed' then abort;
        m_currentLanguageDescription := l_temp;
        m_configSaveFile := f_configSaveFile;
  except
        g_errLog.AddErrStr('TdspLanguage.setConfigSaveFile: f_configSaveFile do not a valid config file');
        l_ini.Free;
        exit;
  end;
  l_ini.Free;
  Result := true;
end;

function TdspLanguage.SaveDescriptionToConfgSaveFile(f_fileDescription: string): boolean;
var
  l_ini: TInifile;
  l_temp: string;
begin
  Result := false;
  if m_configSaveFile = '' then exit;
  l_ini := TIniFile.Create(m_configSaveFile);
  try
        l_ini.WriteString(lc_iniLangFileIdent,lc_iniLangDescriptionIdent,f_fileDescription);
        l_temp := l_ini.ReadString(lc_iniLangFileIdent,lc_iniLangDescriptionIdent,'failed');
        if l_temp = 'failed' then abort;
  except
        g_errLog.AddErrStr('TdspLanguage.SaveDescriptionToConfgSaveFile: an error occoured while writing f_fileDescription');
        l_ini.Free;
        exit;
  end;
  l_ini.Free;
  Result := true;
end;                  

function TdspLanguage.getActiveLanguageAsDescription: string;
begin
  Result := m_currentLanguageDescription;
end;

function TdspLanguage.setLanguageFilePath(f_languageFilePath: string): boolean;
begin
  m_languageFilePath := f_languageFilePath;
  Result := true;
end;

function TdspLanguage.setLanguageFileByDecription(f_fileDescription: String): boolean;
var
  l_filelistbox: TFileListBox;
  l_i: integer;
  l_ini: TInifile;
  l_tempdir: string;
  l_tempvalue: string;
  l_languageFile: string;
  l_form: TForm;
begin
  Result := true;
  l_languageFile := '';
  l_form := TForm.Create(nil);
  l_filelistbox := TFileListBox.Create(l_form);
  l_filelistbox.Parent := l_form;
  l_tempdir := l_filelistbox.Directory;
  l_filelistbox.Directory := m_languageFilePath;
  if l_filelistbox.Items.Count = 0 then begin
        g_errLog.AddErrStr('TdspLanguage.setLanguageFileByDecription: no language files are available');
        Result := false;
  end;
  for l_i := 0 to l_filelistbox.Items.Count-1 do begin
        l_ini := TIniFile.Create(m_languageFilePath+'\'+l_filelistbox.Items[l_i]);
        try
             l_tempvalue := l_ini.ReadString(lc_iniLangFileIdent,lc_iniLangDescriptionIdent,'failed');
             if l_tempvalue = 'failed' then Abort;
             if l_tempvalue = f_fileDescription then
                l_languageFile := m_languageFilePath+'\'+l_filelistbox.Items[l_i];
        except
             g_errLog.AddErrStr('TdspLanguage.setLanguageFileByDecription: a not falid language file has been found: ' +  l_filelistbox.Items[l_i]);
        end;
        l_ini.Free;
  end;
  l_filelistbox.Directory := l_tempdir;
  l_filelistbox.Destroy;
  l_form.Destroy;
  if not (l_languageFile = '') then begin
        m_languageFile := l_languageFile;
        m_currentLanguageDescription := f_fileDescription;
        SaveDescriptionToConfgSaveFile(f_fileDescription);
  end else begin
        g_errLog.AddErrStr('TdspLanguage.setLanguageFileByDecription: there is no file with description: ' + f_fileDescription);
        Result := false;
  end;
end;

function TdspLanguage.getLanguageDescriptions(out f_Descriptions: TStrings): boolean;
var
  l_filelistbox: TFileListBox;
  l_i: integer;
  l_ini: TInifile;
  l_tempdir: string;
  l_tempvalue: string;
  l_Form: TForm;
begin
  Result := true;
  f_Descriptions.Clear;
  l_Form := TForm.Create(nil);
  l_filelistbox := TFileListBox.Create(l_Form);
  l_filelistbox.Parent := l_Form;
  l_tempdir := l_filelistbox.Directory;
  l_filelistbox.Directory := m_languageFilePath;
  if l_filelistbox.Items.Count = 0 then begin
        g_errLog.AddErrStr('TdspLanguage.getLanguageDescriptions: no language files are available');
        Result := false;
  end;
  for l_i := 0 to l_filelistbox.Items.Count-1 do begin
        l_ini := TIniFile.Create(m_languageFilePath+'\'+l_filelistbox.Items[l_i]);
        try
             l_tempvalue := l_ini.ReadString(lc_iniLangFileIdent,lc_iniLangDescriptionIdent,'failed');
             if l_tempvalue = 'failed' then Abort;
             f_Descriptions.Add(l_tempvalue);
        except
             g_errLog.AddErrStr('TdspLanguage.getLanguageDescriptions: a not falid language file has been found: ' +  l_filelistbox.Items[l_i]);
        end;
        l_ini.Free;
  end;
  l_filelistbox.Directory := l_tempdir;
  l_filelistbox.Destroy;
  l_Form.Destroy;
end;

function TdspLanguage.getPopupMenuCaptions(out f_PopupmenuData: TStrings;
                                       f_menuItemNames: TStrings): boolean;
var
  l_ini: TIniFile;
  l_i: integer;
  l_tempValue: string;
begin
  Result := true;
  f_PopupmenuData.Clear;
  l_ini := TIniFile.Create(m_languageFile);
  try
    with l_ini do begin
        for l_i := 0 to f_menuItemNames.Count-1 do begin
                l_tempValue := ReadString(lc_iniPopupmenuSection,f_menuItemNames[l_i],'failed');
                if l_tempValue = 'failed' then begin
                 g_errLog.AddErrStr(f_menuItemNames[l_i]);
                 Abort;
                end;
                f_PopupmenuData.Add(l_tempValue);
        end;
    end;
  except
    g_errLog.AddErrStr('TdspLanguage.getSetupPopupMenuCaptions reading the language file failed');
    Result := false;
  end;
  l_ini.Free;
end;


end.
