unit dspPopupMenu;


    (*********************************************************************
     *  unit dspPopupMenu                                                *
     *                                                                   *
     *  home page : http://www.dsplayer.de                               *
     *  email     : coder@dsplayer.de                                    *
     *                                                                   *
     *                                                                   *
     *  date      : 15-03-2004                                           *
     *                                                                   *
     *  The contents of this file are used with permission, subject to   *
     *  the Mozilla Public License Version 1.1 (the "License"); you may  *
     *  not use this file except in compliance with the License. You may *
     *  obtain a copy of the License at                                  *
     *  http://www.mozilla.org/MPL/MPL-1.1.html                          *
     *                                                                   *
     *  Software distributed under the License is distributed on an      *
     *  "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   *
     *  implied. See the License for the specific language governing     *
     *  rights and limitations under the License.                        *
     *                                                                   *
     *********************************************************************)

{
  @abstract(DSPlayer Components.)
  @author(Martin Offenwanger: coder@dsplayer.de)
  @created(Mar 15, 2004)
  @lastmod(Apr 19, 2004)
}

interface

uses
   Menus, Classes, SysUtils, Graphics, Types, StdCtrls, windows,
   Controls, Messages, Forms, ExtCtrls, Fonts, dspPopupDatatypes, SyncObjs;


type
   TdspPopupMenu = class(TControl)
  private
    m_Owner: TComponent;
    m_popupmenu: TPopupMenu;
    m_CriticalSection: TCriticalSection;
    // array hold all used MenuItems
    m_itenms: array of TMenuItem;
    m_itemswidth: integer;
    m_relativWidth: boolean;
    m_TextRight: integer;
    m_itemsheight: integer;
    m_TextLeft: integer;
    m_blankHeight: integer;
    m_BlankColor: TColor;            
    m_Backcolor: TColor;
    m_fontColor: TColor;
    m_SelectedBackcolor: TColor;
    m_SelectedFontColor: TColor;
    m_checkedColor: TColor;
    m_selectedCheckedColor: TColor;
    m_borderColor: TColor;
    m_disabledColor: TColor;
    m_disabledSelectedColor: TColor;
    m_leftOffset: integer;
    m_iconRightPlaceHolder: integer;
    m_submenuArrowPlaceHolder: integer;
    m_iconBackcolor: TColor;
    m_iconForeColor: TColor;
    m_iconSelectedBackcolor: TColor;
    m_iconSelectedForeColor: TColor;
    m_iconTempBackcolor: TColor;
    m_iconTempForeColor: TColor;
    m_icon3dRightBottomEdgeBordersColor: TColor;
    m_icon3dLeftTopEdgeBordersColor: TColor;
    m_icon3dRightSelectedBottomEdgeBordersColor: TColor;
    m_icon3dLeftSelectedTopEdgeBordersColor: TColor;
    m_SubmenuArrowColor: TColor;
    m_SubmenuArrowSelectedColor: TColor;
    m_submenuSkinningEnabled: boolean;
    m_iconMode: TIconMode;
    m_onMenuItemClick: TOnMenuItemClick;
    m_onMenuPopup: TOnMenuPopup;
    m_Menu_Font: TFonts;
    m_Fontname: string;
    m_fontFile: string;
    m_fontSize: integer;
    m_fontstyles: string;
    m_fontFilePath: string;
    m_IconPath: string;
    m_iconsOn:  boolean;
    // used to count the offset of the drawing items
    m_blankIndex: integer;
    HwndM: HWND;
    m_defFileMenuItemnames: TStrings;
    m_skinningEnabled: boolean;
    procedure setSkinningEnabled(f_skinningEnabled: boolean);
    { EVENTS }
    procedure OnMeasureItem(Sender: TObject; ACanvas: TCanvas; var Width, Height: Integer);
    procedure OnAdvanDrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect; State: TOwnerDrawState);
    procedure OnPopup(Sender: TObject);
    // Border Draw Functions
    procedure DrawWindowBorder(hWnd: HWND);
    function MainWindowHook(var Message: TMessage): boolean;
    procedure getIconImage(f_iconBitmap: Graphics.TBitmap; out f_iconImage: Graphics.TBitmap);
    procedure TColor2RGB(const Color: TColor; var R, G, B: Byte);
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy();
    procedure addItem(f_ItemName: string;f_ItemCaption: string;f_OwnerItemName: string = '';f_iconFile: string = '';f_GroupID: integer = 0);
    procedure changeCaption(f_Itemname: string; f_NewCaption: string);
    procedure setAutoCheck(f_enable: boolean; f_ItemName: string);
    procedure setRadio(f_enable: boolean; f_ItemName: string);
    procedure setChecked(f_checked: boolean; f_ItemName: string);
    procedure setFontfilePath(f_path: string);
    procedure setFont(f_fontFile: string; f_fontSize: integer; f_fontstyles: string = '');
    function getFontFile: string;
    function getFontName: String;
    function getFontSize: integer;
    function getFontStyles: string;
    procedure RemoveSubMenu(f_MenuItemName: string);
    function getItemByName(f_ItemName: string): TMenuItem;
    function getAsTPopupMenu: TPopupmenu;
    // automatic MenuItemCreation by using a def File
    // !! This Parser "generateMenueEntriesByDefFile" needs some more work !!
    function generateMenueEntriesByDefFile(f_defFile: string; out f_itemNames: TStrings): boolean;
    // only use after calling generateMenueEntriesByDefFile
    function getDefFileMenuItemNames(): TStrings;
    function setMenuEntryCaptions(f_captions: TStrings): boolean;
    { EVENTS }
    // all MenuItem-click-events inside the class are forwarded to this function
    procedure onClick(sender: TObject);
    // Load Save support for the skinning config
    procedure saveCurrentSettingsToDsfFile(f_file: string);
    procedure loadCurrentSettingsFromDsfFile(f_file: string);
  published
    property OnMenuItemClick: TOnMenuItemClick read m_onMenuItemClick write m_onMenuItemClick;
    property OnMenuPopup: TOnMenuPopup read m_onMenuPopup write m_onMenuPopup;
    property BlankColor: TColor read m_BlankColor write m_BlankColor;
    property Backcolor: TColor read m_Backcolor write m_Backcolor;
    property FontColor: TColor read m_fontColor write m_fontColor;
    property SelectedBackColor: TColor read m_SelectedBackcolor write m_SelectedBackcolor;
    property SelectedFontColor: TColor read m_SelectedFontColor write m_SelectedFontColor;
    property CheckedColor: TColor read m_checkedColor write m_checkedColor;
    property SelectedCheckedColor: TColor read m_selectedCheckedColor write m_selectedCheckedColor;
    property BorderColor: TColor read m_borderColor write m_borderColor;
    property TextLeft: integer read m_TextLeft write m_TextLeft;
    property TextRight: integer read m_TextRight write m_TextRight;
    property BlankHeigt: integer read m_blankHeight write m_blankHeight;
    property ItemsHeight: integer read m_itemsheight write m_itemsheight;
    property ItemsWidth: integer read m_itemswidth write m_itemswidth;
    property RelativWidth: boolean read m_relativWidth write m_relativWidth;
    property IconPath: string read m_IconPath write m_IconPath;
    property leftOffset: Integer read m_leftOffset write m_leftOffset;
    property IconRightPlaceHolder: Integer read m_iconRightPlaceHolder write m_iconRightPlaceHolder;
    property IconBackColor: TColor read m_iconBackcolor write m_iconBackcolor;
    property IconForeColor: TColor read m_iconForeColor write m_iconForeColor;
    property IconSelectedBackColor: TColor read m_iconSelectedBackcolor write m_iconSelectedBackcolor;
    property IconSelectedForeColor: TColor read m_iconSelectedForeColor write m_iconSelectedForeColor;
    property Icon3dRightBottomEdgeBordersColor: TColor read m_icon3dRightBottomEdgeBordersColor write m_icon3dRightBottomEdgeBordersColor;
    property Icon3dLeftTopEdgeBordersColor: TColor read m_icon3dLeftTopEdgeBordersColor write m_icon3dLeftTopEdgeBordersColor;
    property Icon3dRightSelectedBottomEdgeBordersColor: TColor read m_icon3dRightSelectedBottomEdgeBordersColor write m_icon3dRightSelectedBottomEdgeBordersColor;
    property Icon3dLeftSelectedTopEdgeBordersColor: TColor read m_icon3dLeftSelectedTopEdgeBordersColor write m_icon3dLeftSelectedTopEdgeBordersColor;
    property IconMode: TIconMode read m_iconMode write m_iconMode;
    property IconsOn: Boolean read m_iconsOn write m_iconsOn;
    property SubmenuArrowPlaceHolder: Integer read m_submenuArrowPlaceHolder write m_submenuArrowPlaceHolder;
    property SubmenuArrowColor: TColor read m_SubmenuArrowColor write m_SubmenuArrowColor;
    property SubmenuArrowSelectedColor: TColor read m_SubmenuArrowSelectedColor write m_SubmenuArrowSelectedColor;
    property DisabledColor: TColor read m_disabledColor write m_disabledColor;
    property DisabledSelectedColor: TColor read m_disabledSelectedColor write m_disabledSelectedColor;
    property SkinningEnabled: boolean read m_skinningEnabled write setSkinningEnabled;
    property SubmenuSkinningEnabled: boolean read m_submenuSkinningEnabled write m_submenuSkinningEnabled;
end;

implementation

uses Globals;

const
  //-----------------------------------------------
  // when setting the width on measure item,
  // the advanced draw item always automaticly
  // add 12 pixels, because of submenu arrow.
  // however, i want to control this width
  // exactly by myself, so I added this inconsisty
  // value, to remove this automatic added 12 value
  lc_MeasureDrawItemInconsistenz = 12;
  //-----------------------------------------------
  WM_DRAWMENUBORDER             = CN_NOTIFY + 101;
  WM_ARROW                      = CN_NOTIFY + 102;
  // Item Goup const (holed in TMenuItem helpContext)
  LCICONGROUP                   = 1;
  LCICONSUBMENUGROUP            = 2;
  LCSUBMENUGROUP                = 3;
  // def for Mainwindow Hook operations
  LCSELECTED                    = 4;
  LCDISABLED                    = 5;
  LCDISABLEDSELECTED            = 6;
  // Def file parser Commands
  lc_iconGroupBegin             = '++icongroup begin';
  lc_iconGroupEnd               = '++icongroup end';
  lc_iconSubmenuGroupBegin      = '++iconSubmenugroup begin';
  lc_iconSubmenuGroupEnd        = '++iconSubmenugroup end';
  lc_submenuGroupBegin          = '++submenugroup begin';
  lc_submenuGroupEnd            = '++submenugroup end';
  lc_defPopupStartTag           = '[popupmenu]';
  lc_defPopupBlankItem          = 'blank';
  lc_defPopupSubMenu            = '**submenu';
  lc_defPopupRadioItem          = '**radioitems';
  lc_defPopupAutoCheck          = '**autocheck';
  lc_defCommandPlaceHolder      = '**';
  lc_iconTag                    = '**icon';
  lc_defPopupEndTag             = '[end]';
  // dsf file start tag
  lc_dsfFileStartTag            = 'DSPlayer dsf file for Popup config';
  // getIconImage - binary colors
  lc_iconBinBackColor = clwhite;
  lc_iconBinForeColor = clBlack;

procedure TdspPopupMenu.loadCurrentSettingsFromDsfFile(f_file: string);
var
  l_file: TextFile;
  l_temp: string;
  l_temp2: string;
  l_temp3: string;
  r,g,b:  Byte;
  procedure readThreeLines;
  begin
        Readln(l_file,l_temp);
        Readln(l_file,l_temp2);
        Readln(l_file,l_temp3);
  end;
  function SrtingToBoolean(f_bool: string): boolean;
  begin
        if f_bool = 'TRUE' then begin
                Result := true;
                exit;
        end else begin
                Result := false;
        end;
  end;
begin
  g_errLog.AddErrStr('TdspPopupMenu.loadCurrentSettingsFromDsfFile');
  try
        AssignFile(l_file,f_file);
        Reset(l_file);
        // read thr firstline
        Readln(l_file,l_temp);
        if not (l_temp = lc_dsfFileStartTag) then abort;
        // read the bordercolor values to file
        readThreeLines;
        BorderColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the BlankColor values to file
        readThreeLines;
        BlankColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the Backcolor values to file
        readThreeLines;
        Backcolor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the SelectedBackColor values to file
        readThreeLines;
        SelectedBackColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the FontColor values to file
        readThreeLines;
        FontColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the SelectedFontColor values to file
        readThreeLines;
        SelectedFontColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the CheckedColor values to file
        readThreeLines;
        CheckedColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the SelectedCheckedColor values to file
        readThreeLines;
        SelectedCheckedColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the IconBackColor values to file
        readThreeLines;
        IconBackColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the IconForeColor values to file
        readThreeLines;
        IconForeColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the IconSelectedBackColor values to file
        readThreeLines;
        IconSelectedBackColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the IconSelectedForeColor values to file
        readThreeLines;
        IconSelectedForeColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the Icon3dRightBottomEdgeBordersColor values to file
        readThreeLines;
        Icon3dRightBottomEdgeBordersColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the Icon3dLeftTopEdgeBordersColor values to file
        readThreeLines;
        Icon3dLeftTopEdgeBordersColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the Icon3dRightSelectedBottomEdgeBordersColor values to file
        readThreeLines;
        Icon3dRightSelectedBottomEdgeBordersColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the Icon3dLeftSelectedTopEdgeBordersColor values to file
        readThreeLines;
        Icon3dLeftSelectedTopEdgeBordersColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the DisabledColor values to file
        readThreeLines;
        DisabledColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the DisabledSelectedColor values to file
        readThreeLines;
        DisabledSelectedColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the SubmenuArrowColor values to file
        readThreeLines;
        SubmenuArrowColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the SubmenuArrowSelectedColor values to file
        readThreeLines;
        SubmenuArrowSelectedColor := RGB(strtoint(l_temp),strtoint(l_temp2),strtoint(l_temp3));
        // read the TextLeft value to file
        readln(l_file,l_temp);
        TextLeft := StrToInt(l_temp);
        // read the TextRight value to file
        readln(l_file,l_temp);
        TextRight := StrToInt(l_temp);
        // read the BlankHeigt value to file
        readln(l_file,l_temp);
        BlankHeigt := StrToInt(l_temp);
        // read the ItemsHeight value to file
        readln(l_file,l_temp);
        ItemsHeight := StrToInt(l_temp);
        // read the ItemsWidth value to file
        readln(l_file,l_temp);
        ItemsWidth := StrToInt(l_temp);
        // read the leftOffset value to file
        readln(l_file,l_temp);
        leftOffset := StrToInt(l_temp);
        // read the IconRightPlaceHolder value to file
        readln(l_file,l_temp);
        IconRightPlaceHolder := StrToInt(l_temp);
        // read the SubmenuArrowPlaceHolder value to file
        readln(l_file,l_temp);
        SubmenuArrowPlaceHolder := StrToInt(l_temp);
        // read the RelativWidth value to file
        readln(l_file,l_temp);
        RelativWidth := SrtingToBoolean(l_temp);
        // read the SkinningEnabled value to file
        readln(l_file,l_temp);
        setSkinningEnabled(SrtingToBoolean(l_temp));
        // read the SubmenuSkinningEnabled value to file
        readln(l_file,l_temp);
        SubmenuSkinningEnabled := SrtingToBoolean(l_temp);
        if SubmenuSkinningEnabled then
                g_errLog.AddErrStr('TdspPopupMenu.loadCurrentSettingsFromDsfFile  enabled')
        else
                g_errLog.AddErrStr('TdspPopupMenu.loadCurrentSettingsFromDsfFile  disabled');
        // read the IconsOn value to file
        readln(l_file,l_temp);
        IconsOn := SrtingToBoolean(l_temp);
        // read the IconMode value to file
        readln(l_file,l_temp);
        if l_temp = '[none]' then
                IconMode := [];
        if l_temp = '[icmLineIncluded]' then
                IconMode := [icmLineIncluded];
        if l_temp = '[icmSeperateColors]' then
                IconMode := [icmSeperateColors];
        if l_temp = '[icm3DBorder]' then
                IconMode := [icm3DBorder];
        if l_temp = '[icmLineIncluded,icmSeperateColors]' then
                IconMode := [icmLineIncluded,icmSeperateColors];
        if l_temp = '[icmLineIncluded,icm3DBorder]' then
                IconMode := [icmLineIncluded,icm3DBorder];
        if l_temp = '[icmSeperateColors,icm3DBorder]' then
                IconMode := [icmSeperateColors,icm3DBorder];
        if l_temp = '[icmLineIncluded,icmSeperateColors,icm3DBorder]' then
                IconMode := [icmLineIncluded,icmSeperateColors,icm3DBorder];
        readln(l_file,l_temp);
        readln(l_file,l_temp2);
        readln(l_file,l_temp3);
        setFont(l_temp,strtoint(l_temp2),l_temp3);
        // close the file
        CloseFile(l_file);
  except
        g_errLog.AddErrStr('TdspPopupMenu.loadCurrentSettingsFromDsfFile: an unknown error occourt while writing the file');
  end;
end;

procedure TdspPopupMenu.saveCurrentSettingsToDsfFile(f_file: string);
var
  l_file: TextFile;
  r,g,b:  Byte;
  procedure writeColorValues(f_color: TColor);
  begin
        TColor2RGB(f_color,r,g,b);
        writeln(l_file,r);
        writeln(l_file,g);
        writeln(l_file,b);
  end;
begin
  try
        AssignFile(l_file,f_file);
        Rewrite(l_file);
        // write the start Tag
        writeln(l_file,lc_dsfFileStartTag);
        // write the bordercolor values to file
        writeColorValues(BorderColor);
        // write the BlankColor values to file
        writeColorValues(BlankColor);
        // write the Backcolor values to file
        writeColorValues(Backcolor);
        // write the SelectedBackColor values to file
        writeColorValues(SelectedBackColor);
        // write the FontColor values to file
        writeColorValues(FontColor);
        // write the SelectedFontColor values to file
        writeColorValues(SelectedFontColor);
        // write the CheckedColor values to file
        writeColorValues(CheckedColor);
        // write the SelectedCheckedColor values to file
        writeColorValues(SelectedCheckedColor);
        // write the IconBackColor values to file
        writeColorValues(IconBackColor);
        // write the IconForeColor values to file
        writeColorValues(IconForeColor);
        // write the IconSelectedBackColor values to file
        writeColorValues(IconSelectedBackColor);
        // write the IconSelectedForeColor values to file
        writeColorValues(IconSelectedForeColor);
        // write the Icon3dRightBottomEdgeBordersColor values to file
        writeColorValues(Icon3dRightBottomEdgeBordersColor);
        // write the Icon3dLeftTopEdgeBordersColor values to file
        writeColorValues(Icon3dLeftTopEdgeBordersColor);
        // write the Icon3dRightSelectedBottomEdgeBordersColor values to file
        writeColorValues(Icon3dRightSelectedBottomEdgeBordersColor);
        // write the Icon3dLeftSelectedTopEdgeBordersColor values to file
        writeColorValues(Icon3dLeftSelectedTopEdgeBordersColor);
        // write the DisabledColor values to file
        writeColorValues(DisabledColor);
        // write the DisabledSelectedColor values to file
        writeColorValues(DisabledSelectedColor);
        // write the SubmenuArrowColor values to file
        writeColorValues(SubmenuArrowColor);
        // write the SubmenuArrowSelectedColor values to file
        writeColorValues(SubmenuArrowSelectedColor);
        // write the TextLeft value to file
        writeln(l_file,TextLeft);
        // write the TextRight value to file
        writeln(l_file,TextRight);
        // write the BlankHeigt value to file
        writeln(l_file,BlankHeigt);
        // write the ItemsHeight value to file
        writeln(l_file,ItemsHeight);
        // write the ItemsWidth value to file
        writeln(l_file,ItemsWidth);
        // write the leftOffset value to file
        writeln(l_file,leftOffset);
        // write the IconRightPlaceHolder value to file
        writeln(l_file,IconRightPlaceHolder);
        // write the SubmenuArrowPlaceHolder value to file
        writeln(l_file,SubmenuArrowPlaceHolder);
        // write the RelativWidth value to file
        writeln(l_file,RelativWidth);
        // write the SkinningEnabled value to file
        writeln(l_file,SkinningEnabled);
        // write the SubmenuSkinningEnabled value to file
        writeln(l_file,SubmenuSkinningEnabled);
        // write the IconsOn value to file
        writeln(l_file,IconsOn);
        // write the IconMode value to file
        if IconMode = [] then
                writeln(l_file,'[none]');
        if IconMode = [icmLineIncluded] then
                writeln(l_file,'[icmLineIncluded]');
        if IconMode = [icmSeperateColors] then
                writeln(l_file,'[icmSeperateColors]');
        if IconMode = [icm3DBorder] then
                writeln(l_file,'[icm3DBorder]');
        if IconMode = [icmLineIncluded,icmSeperateColors] then
                writeln(l_file,'[icmLineIncluded,icmSeperateColors]');
        if IconMode = [icmLineIncluded,icm3DBorder] then
                writeln(l_file,'[icmLineIncluded,icm3DBorder]');
        if IconMode = [icmSeperateColors,icm3DBorder] then
                writeln(l_file,'[icmSeperateColors,icm3DBorder]');
        if IconMode = [icmLineIncluded,icmSeperateColors,icm3DBorder] then
                writeln(l_file,'[icmLineIncluded,icmSeperateColors,icm3DBorder]');
        // write the Fontfile value
        writeln(l_file,m_fontFile);
        writeln(l_file,m_fontSize);
        writeln(l_file,m_fontstyles);
        // close the file
        closeFile(l_file);
  except
        g_errLog.AddErrStr('TdspPopupMenu.SaveCurrentSettings: an unknown error occourt while writing the file');
  end;
end;

procedure TdspPopupMenu.setSkinningEnabled(f_skinningEnabled: boolean);
begin
m_popupmenu.OwnerDraw := f_skinningEnabled;
m_skinningEnabled := m_popupmenu.OwnerDraw;
end;


function TdspPopupMenu.getDefFileMenuItemNames(): TStrings;
begin
 Result := m_defFileMenuItemnames;
end;

procedure TdspPopupMenu.TColor2RGB(const Color: TColor; var R, G, B: Byte);
begin
  // convert hexa-decimal values to RGB
  R := Color and $FF;
  G := (Color shr 8) and $FF;
  B := (Color shr 16) and $FF;
end;

procedure TdspPopupMenu.getIconImage(f_iconBitmap: Graphics.TBitmap; out f_iconImage: Graphics.TBitmap);
type
  PixArray = Array [1..3] of Byte;
var
  l_p1: ^PixArray;
  l_p2: ^PixArray;
  l_h,l_w: Integer;
  l_found: boolean;
  // icon binary BackGround color values
  l_ibb_r,l_ibb_g,l_ibb_b: byte;
  // icon binary ForeGround color values
  l_ibf_r,l_ibf_g,l_ibf_b: byte;
  // icon BackColor Values
  l_ib_r,l_ib_g,l_ib_b: byte;
  // icon ForeColor Values
  l_if_r,l_if_g,l_if_b: byte;
begin
  f_iconBitmap.PixelFormat :=  pf24Bit;
  f_iconImage.PixelFormat  :=  f_iconBitmap.PixelFormat;
  f_iconImage.Height       := f_iconBitmap.Height;
  f_iconImage.Width        := f_iconBitmap.Width;
  // get the binary background color
  TColor2RGB(lc_iconBinBackColor,l_ibb_r,l_ibb_g,l_ibb_b);
  // get the binary foreground color
  TColor2RGB(lc_iconBinForeColor,l_ibf_r,l_ibf_g,l_ibf_b);
  // get the background color
  TColor2RGB(m_iconTempBackcolor,l_ib_r,l_ib_g,l_ib_b);
  // get the binary foreground color
  TColor2RGB(m_iconTempForeColor,l_if_r,l_if_g,l_if_b);
  for l_h:=0 to f_iconBitmap.Height-1 do begin
        l_p1:= f_iconBitmap.ScanLine[l_h];
        l_p2:= f_iconImage.ScanLine[l_h];
        for l_w:=0 to f_iconBitmap.Width-1 do begin
                l_found := false;
                if (l_p1^[1]=l_ibb_b) and (l_p1^[2]=l_ibb_g) and (l_p1^[3]=l_ibb_r) then begin
                      l_p2^[1] := l_ib_b;
                      l_p2^[2] := l_ib_g;
                      l_p2^[3] := l_ib_r;
                      //  g_errLog.AddErrStr('found white pixel');
                      l_found := true;
                end else
                        if (l_p1^[1]=l_ibf_b) and (l_p1^[2]=l_ibf_g) and (l_p1^[3]=l_ibf_r) then begin
                              l_p2^[1] := l_if_b;
                              l_p2^[2] := l_if_g;
                              l_p2^[3] := l_if_r;
                                // g_errLog.AddErrStr('found black pixel');
                                l_found := true;
                        end;
                if not l_found then
                        g_errLog.AddErrStr('image do not contain only binary data');
                inc(l_p1);
                inc(l_p2);
        end;
  end;
end;

function TdspPopupMenu.generateMenueEntriesByDefFile(f_defFile: string;out f_itemNames: TStrings): boolean;
var
  l_file: Textfile;
  l_temp: string;
  l_iconFile: string;
  l_found: boolean;
  l_submenu: string;
  l_radioitem: boolean;
  l_autocheck: boolean;
  l_tempFileMode: byte;
  l_i: integer;
  l_groupID: Integer;
begin
 Result := false;
 f_itemNames.Clear;
 l_submenu := '';
 l_radioitem := false;
 l_autocheck := false;
 l_groupID   := 0;
 if not FileExists(f_defFile) then exit;
 Result := true;
 try
        l_temp := 'empty';
        AssignFile(l_file,f_defFile);
        l_tempFileMode := FileMode;
        FileMode := fmOpenRead;
        Reset(l_file);
        FileMode := l_tempFileMode;
        l_found := false;
        while not (l_temp =  lc_defPopupStartTag) do begin
                If eof(l_file) then Abort;
                Readln(l_file, l_temp);
                if (l_temp = lc_defPopupStartTag) then
                        l_found := true;
        end;
        if l_found= false then Abort;
        Readln(l_file,l_temp);
        while not (l_temp = lc_defPopupEndTag) do begin
                if eof(l_file) then Abort;
                if l_temp = lc_iconGroupBegin then begin
                        l_groupID := LCICONGROUP;
                        Readln(l_file,l_temp);
                end;
                if l_temp = lc_iconSubmenuGroupBegin then begin
                        l_groupID := LCICONSUBMENUGROUP;
                        Readln(l_file,l_temp);
                end;
                if l_temp = lc_submenuGroupBegin then begin
                        l_groupID := LCSUBMENUGROUP;
                        Readln(l_file,l_temp);
                end;
                if (l_temp = lc_iconSubmenuGroupEnd) or
                   (l_temp = lc_iconGroupEnd) or
                   (l_temp = lc_submenuGroupEnd) then begin
                        l_groupID := 0;
                        Readln(l_file,l_temp);
                end;
                 if l_temp = lc_iconGroupBegin then begin
                        l_groupID := LCICONGROUP;
                        Readln(l_file,l_temp);
                end;
                if l_temp = lc_iconSubmenuGroupBegin then begin
                        l_groupID := LCICONSUBMENUGROUP;
                        Readln(l_file,l_temp);
                end;
                if l_temp = lc_submenuGroupBegin then begin
                        l_groupID := LCSUBMENUGROUP;
                        Readln(l_file,l_temp);
                end;
                if (l_temp = lc_iconSubmenuGroupEnd) or
                   (l_temp = lc_iconGroupEnd) or
                   (l_temp = lc_submenuGroupEnd) then begin
                        l_groupID := 0;
                        Readln(l_file,l_temp);
                end;
                l_iconFile := '';
                if l_temp = lc_iconTag then begin
                        Readln(l_file,l_temp);
                        if Copy(l_temp,1,2) <> lc_defCommandPlaceHolder then begin
                                g_errLog.AddErrStr('TdspPopupMenu.generateMenueEntriesByDefFile: missing icon file definition in def file');
                                abort;
                        end;
                        l_iconFile := Copy(l_temp,3,length(l_temp));
                        Readln(l_file,l_temp);
                end;
                if l_temp = lc_defPopupRadioItem then begin
                        l_radioitem := true;
                        Readln(l_file,l_temp);
                end;
                if l_temp = lc_defPopupAutoCheck then begin
                        l_autocheck := true;
                        Readln(l_file,l_temp);
                end;
                if l_temp = lc_defPopupSubMenu then begin
                        l_radioitem := false;
                        l_autocheck := false;
                        Readln(l_file,l_submenu);
                        l_submenu := copy(l_submenu,3,length(l_submenu));
                        while true do begin
                                Readln(l_file,l_temp);
                                if l_temp = lc_defPopupRadioItem then
                                        l_radioitem := true;
                                if l_temp = lc_defPopupAutoCheck then
                                        l_autocheck := true;
                                if Copy(l_temp,1,2) <> lc_defCommandPlaceHolder then break;
                        end;
                end;
                if copy(l_temp,1,length(l_temp)-1) = lc_defPopupBlankItem then
                        addItem(l_temp,'-',l_submenu)
                else begin                   
                        f_itemNames.Add(l_temp);  
                        addItem(l_temp,l_temp,l_submenu,l_iconFile,l_groupID);
                        if l_autocheck then
                                setAutoCheck(true,l_temp);
                        if l_radioitem then
                                setRadio(true,l_temp);
                end;                
                Readln(l_file,l_temp);     
        end;
 except
   g_errLog.AddErrStr('TdspPopupMenu.generateMenueEntriesByDefFile an' +
                      ' unknown error occoured');
   CloseFile(l_file);
   exit;
 end;
 CloseFile(l_file);
 m_defFileMenuItemnames.Clear;
 for l_i := 0 to f_itemNames.Count-1 do
        m_defFileMenuItemnames.Add(f_itemNames[l_i]);
end;

function TdspPopupMenu.setMenuEntryCaptions(f_captions: TStrings): boolean;
var
  l_i: integer;
  l_j: integer;
begin
  Result := true;
  l_j := 0;
  // If MenuItems are generated by def file
  // we get the count from m_defFileMenuItemnames
  // --> only the Caption with the names from m_defFileMenuItemnames will be changed
  if m_defFileMenuItemnames.Count > 0 then begin
        for l_i := 0 to high(m_itenms) do
                // we do not reset blank-item caption
                if not (m_itenms[l_i].Caption = '-') then begin
                        m_itenms[l_i].Caption := f_captions[l_j];
                        inc(l_j);
                        if l_j =  m_defFileMenuItemnames.Count then break;
                end;
  end else begin
        for l_i := 0 to high(m_itenms) do
                // we do not reset blank-item caption
                if not (m_itenms[l_i].Caption = '-') then begin
                        m_itenms[l_i].Caption := f_captions[l_j];
                        inc(l_j);
                end;
  end;
end;

procedure TdspPopupMenu.RemoveSubMenu(f_MenuItemName: string);
var
  l_i: Integer;
  l_j: Integer;
  l_k: Integer;
  l_highItems: Integer;
  l_MenuItem: TMenuItem;
begin
  l_MenuItem := getItemByName(f_MenuItemName);
  // delete from popupmenu and dynamic array
  if l_MenuItem <> nil then
        for l_i := 0 to l_MenuItem.Count - 1 do begin
               for l_j := 0 to high(m_itenms) do
                        if m_itenms[l_j].Name = l_MenuItem.Items[0].Name then begin
                                // Remove from popupmenu item
                                l_MenuItem.Remove(l_MenuItem.Items[0]);
                                // remove from dynamic array
                                m_itenms[l_j].Destroy;
                                // resort the array
                                for l_k := l_j to high(m_itenms)-1 do
                                        m_itenms[l_k] := m_itenms[l_k+1];
                                setlength(m_itenms,high(m_itenms));
                                break;
                        end;
        end;
end;

destructor TdspPopupMenu.destroy();
begin
inherited Destroy;
m_popupmenu.Destroy;
if m_Menu_Font = nil then
        g_errLog.AddErrStr('TdspPopupMenu.destroy: Error: There is no Menu Font Defined')
else
        m_Menu_Font.Destroy;
end;

function TdspPopupMenu.getFontFile: string;
begin
 Result := m_fontFile;
end;

function TdspPopupMenu.getFontSize: integer;
begin
 Result := m_fontSize;
end;

function TdspPopupMenu.getFontStyles: string;
begin
 Result := m_fontstyles;
end;

function TdspPopupMenu.getFontName: String;
begin
 Result := m_Fontname;
end;

procedure TdspPopupMenu.setFontfilePath(f_path: string);
begin
m_fontFilePath := f_path;
end;


procedure TdspPopupMenu.setFont(f_fontFile: string; f_fontSize: integer; f_fontstyles: string);
begin
  m_fontFile := f_fontFile;
  m_fontSize := f_fontSize;
  m_fontstyles := f_fontstyles;
  if m_Menu_Font <> nil then
        m_Menu_Font.Destroy;
  m_Menu_Font := TFonts.Create(m_fontFilePath + '\' + f_fontFile);
  m_Fontname :=  m_Menu_Font.GetFontName;
  m_Menu_Font.SetSize(f_fontSize);
  if not (f_fontstyles = '') then
        m_Menu_Font.SetFontStyles(f_fontstyles);
end;

constructor TdspPopupMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  m_Owner := AOwner;
  m_defFileMenuItemnames := TStringList.Create;
  m_relativWidth := true;
  m_BlankColor := clgray;
  m_Backcolor := rgb(231,231,231);
  m_fontColor := clblack;
  m_SelectedBackcolor := clblack;
  m_SelectedFontColor := clWhite;
  m_checkedColor := clblack;
  m_selectedCheckedColor := clwhite;
  m_borderColor := clblack;
  m_leftOffset := 3;
  m_iconRightPlaceHolder := 6;
  m_iconBackcolor := m_Backcolor;
  m_iconForeColor := rgb(55,55,55);
  m_iconSelectedBackcolor := rgb(200,200,200);
  m_iconSelectedForeColor := m_fontColor;
  m_icon3dRightBottomEdgeBordersColor := m_Backcolor;
  m_icon3dLeftTopEdgeBordersColor := m_Backcolor;
  m_icon3dRightSelectedBottomEdgeBordersColor := clBlack;
  m_icon3dLeftSelectedTopEdgeBordersColor := clwhite;
  m_iconMode := [icmSeperateColors,icm3DBorder];
  m_iconsOn := true;
  m_SubmenuArrowColor := RGB(55,55,55);
  m_SubmenuArrowSelectedColor := RGB(231,231,231);
  m_submenuArrowPlaceHolder := 18;
  m_disabledColor := RGB(120,120,120);
  m_disabledSelectedColor := RGB(190,190,190);
  m_itemswidth := 100;
  m_itemsheight := 14;
  m_TextLeft := 3;
  m_TextRight := 6;
  m_blankHeight := 8;
  m_blankIndex := 0;
  m_popupmenu := TPopupMenu.Create(AOwner);
  m_popupmenu.OwnerDraw := true;
  m_skinningEnabled := m_popupmenu.OwnerDraw;
  m_submenuSkinningEnabled := true;
  m_popupmenu.OnPopup := OnPopup;
  m_CriticalSection := TCriticalSection.Create;
  Application.HookMainWindow(MainWindowHook);
  //m_control.WindowProc(
  m_Menu_Font := nil;
  // set an empty array
  setlength(m_itenms,0);
end;

function TdspPopupMenu.getAsTPopupMenu: TPopupmenu;
begin
  Result := m_popupmenu;
end;

procedure TdspPopupMenu.addItem(f_ItemName: string;f_ItemCaption: string;f_OwnerItemName: string = '';f_iconFile: string = '';f_GroupID: integer = 0);
var
  l_menuItem: TMenuItem;
begin
  l_menuItem := getItemByName(f_ItemName);
  if (l_menuItem <> nil) then begin
        g_errLog.AddErrStr('TdspPopupMenu.addItem: '+ 'Itemname: "'+ f_ItemName + '"  already exits');
        exit;
  end;
  if (f_OwnerItemName <> '') then begin
        l_menuItem := getItemByName(f_OwnerItemName);
        if (l_menuItem = nil) then begin
                g_errLog.AddErrStr('TdspPopupMenu.addItem: '+ 'there is no spcific item called: ' + f_ItemName);
                exit;
        end;
  end;
  setlength(m_itenms,(high(m_itenms)+2));
  m_itenms[high(m_itenms)] := TMenuItem.Create(self);
  m_itenms[high(m_itenms)].Name := f_ItemName;
  m_itenms[high(m_itenms)].Caption := f_ItemCaption;
  // the IconFile-Path will be stored in the hint filed,
  // because I see no reason to display hints in a popupmenu
  m_itenms[high(m_itenms)].Hint := f_iconFile;
  // Help Context contains the Item GroupID properties
  // because I see no reason to add a Help possibility for a popup
  m_itenms[high(m_itenms)].HelpContext := f_GroupID;
  // set the events handler
  m_itenms[high(m_itenms)].OnClick := onClick;
  m_itenms[high(m_itenms)].OnMeasureItem := OnMeasureItem;
  m_itenms[high(m_itenms)].OnAdvancedDrawItem := OnAdvanDrawItem;
  // add the Item to root of the Menu
  if f_OwnerItemName = '' then
        m_popupmenu.Items.Insert(high(m_itenms),m_itenms[high(m_itenms)])
  else
        l_menuItem.Add(m_itenms[high(m_itenms)]);
end;

function TdspPopupMenu.getItemByName(f_ItemName: string): TMenuItem;
var
  l_i: integer;
begin
  Result := nil;
  for l_i := 0 to high(m_itenms) do
        if m_itenms[l_i].Name = f_ItemName then begin
                RESULT := m_itenms[l_i];
                break;
        end;
end;

procedure TdspPopupMenu.changeCaption(f_Itemname: string; f_NewCaption: string);
var
  l_MenuItem: TMenuItem;
begin
  l_MenuItem := getItemByName(f_Itemname);
  if l_MenuItem <> nil then
        l_MenuItem.Caption := f_NewCaption
end;

procedure TdspPopupMenu.setAutoCheck(f_enable: boolean; f_ItemName: string);
var
  l_MenuItem: TMenuItem;
begin
  l_MenuItem := getItemByName(f_ItemName);
  if (l_MenuItem <> nil) then
        l_MenuItem.AutoCheck := f_enable;
end;

procedure TdspPopupMenu.setChecked(f_checked: boolean; f_ItemName: string);
var
  l_MenuItem: TMenuItem;
begin
  l_MenuItem := getItemByName(f_ItemName);
  if (l_MenuItem <> nil) then
        l_MenuItem.Checked := f_checked;
end;


procedure TdspPopupMenu.setRadio(f_enable: boolean; f_ItemName: string);
var
  l_MenuItem: TMenuItem;
begin
  l_MenuItem := getItemByName(f_ItemName);
  if (l_MenuItem<> nil) then
        l_MenuItem.RadioItem := f_enable;
end;

{ EVENTS }

procedure TdspPopupMenu.onClick(sender: TObject);
var
  l_MenuItem: TMenuItem;        
begin
  l_MenuItem := TMenuItem(sender);
  if assigned(m_onMenuItemClick) then
        m_onMenuItemClick(self,l_MenuItem.Name);
end;

// Listens to messages sent to the application and looks if a window is inserted.
// I am not shure if "CriticalSection" is needed, so I enclosed it
function TdspPopupMenu.MainWindowHook(var Message: TMessage): boolean;
var
  l_MenuItem: TMenuItem;
  r: TRECT;
  pt: TPOINT;
  hWndM: HWND;
  l_rect: TRect;
  l_Canvas: TCanvas;
  wDC: HDC; // +jt
  l_points: array [0..3] of TPoint;
begin
  Result := false;
  if Message.Msg = WM_DRAWMENUBORDER then
  begin
    m_CriticalSection.Enter;
    l_MenuItem:=TMenuItem(Message.LParam);
    if Assigned(l_MenuItem) then
    begin
      GetMenuItemRect(0,l_MenuItem.Parent.Handle,l_MenuItem.MenuIndex,r);
      pt.x:=r.Left;
      pt.y:=r.Top;
      hWndM :=WindowFromPoint(pt);
      if hWndM<>0 then DrawWindowBorder(hWndM);
    end;
    m_CriticalSection.Leave;
  end;
  if Message.Msg = WM_ARROW then begin
      m_CriticalSection.Enter;
      l_MenuItem:=TMenuItem(Message.LParam);
      GetMenuItemRect(0,l_MenuItem.Parent.Handle,l_MenuItem.MenuIndex,r);
      wDC := GetWindowDC(HWND_DESKTOP);
      l_Canvas := TCanvas.Create;

      l_Canvas.handle := wDC;

      r.Left := r.Right-10;
      r.Right := r.Right - 6;
        if (Message.WParam = LCSELECTED) or (Message.WParam = LCDISABLEDSELECTED) then begin
                l_Canvas.Brush.Color := m_SelectedBackcolor;
                l_Canvas.pen.Color := m_SelectedBackcolor;
        end else begin
                l_Canvas.Brush.Color := m_Backcolor;
                l_Canvas.pen.Color := m_Backcolor;
        end;
      l_Canvas.FillRect(r);
      l_rect.Left := r.Left;
      l_rect.Right := r.Right-1;
      l_rect.Top := round((r.Bottom-r.Top)/2)+r.Top - 4;
      l_rect.Bottom := round((r.Bottom-r.Top)/2)+r.Top + 2;
      if not ((Message.WParam = LCDISABLED) or (Message.WParam = LCDISABLEDSELECTED)) then begin
        if Message.WParam = LCSELECTED then begin
                l_Canvas.Brush.Color := m_SubmenuArrowSelectedColor;
                l_Canvas.pen.Color := m_SubmenuArrowSelectedColor;
        end else begin
                l_Canvas.Brush.Color := m_SubmenuArrowColor;
                l_Canvas.pen.Color := m_SubmenuArrowColor;
        end;
      end else begin
        if Message.WParam = LCDISABLED then begin
                l_Canvas.Brush.Color := m_disabledColor;
                l_Canvas.pen.Color := m_disabledColor;
        end else begin
                l_Canvas.Brush.Color := m_disabledSelectedColor;
                l_Canvas.pen.Color := m_disabledSelectedColor;
        end;
      end;
      l_points[0] := Point( l_rect.Left , l_rect.Top );
      l_points[1] := Point( l_rect.Left , l_rect.Bottom);
      l_points[2] := Point( l_rect.Right, round((l_rect.Bottom-l_rect.Top)/2)+l_rect.Top );
      l_points[3] := Point( l_rect.Left , l_rect.Top );
      l_Canvas.Polygon(l_points);
      ReleaseDC(HWND_DESKTOP, wDC); // +jt
      l_Canvas.Free;
      m_CriticalSection.Leave;
  end;
end;


procedure TdspPopupMenu.DrawWindowBorder(hWnd: HWND);
var
  WRect: TRect;
  l_rect: TRect;
  dCanvas: TCanvas;
  wDC: HDC; // +jt
begin
  wDC := GetWindowDC(hWnd); //GetDesktopWindow
//  if wDC=0 then exit;
  dCanvas := TCanvas.Create;
  try
    dCanvas.Handle := wDC;
    GetWindowRect(hWnd, WRect);
    l_rect.Right := WRect.Right-WRect.Left;
    l_rect.Bottom := WRect.Bottom-WRect.Top;
    l_rect.Top:=0;
    l_rect.Left:=0;
    dCanvas.Brush.Style := bsClear;
    dCanvas.Pen.Color := m_borderColor;
    dCanvas.Rectangle(l_rect.Left, l_rect.Top, l_rect.Right, l_rect.Bottom);
    dCanvas.Pen.Color := m_Backcolor;
    dCanvas.Pen.Width := 2;
    dCanvas.Rectangle(l_rect.Left+2, l_rect.Top+2, l_rect.Right-1, l_rect.Bottom-1);
  finally
    ReleaseDC(hWnd, wDC); // +jt
    dCanvas.Free;
  end;
end;


procedure TdspPopupMenu.OnMeasureItem(Sender: TObject; ACanvas: TCanvas; var Width, Height: Integer);

var
  l_submenuPlaceholder          : integer;
  l_iconRightPlaceholder        : integer;
  l_iconWidth                   : integer;
  l_pos: integer;
  l_caption: string;
begin
  //g_errLog.AddErrStr('TdspPopupMenu.OnMeasureItem : ' + TMenuItem(Sender).Name);
  m_blankIndex           := 0;
  l_submenuPlaceholder   := 0;
  l_iconRightPlaceholder := 0;
  l_iconWidth            := 0;
  // get the right Caption
  l_Caption := TMenuItem(sender).Caption;
  l_pos := Pos('&',l_Caption);
  if l_pos > 0 then
        Delete(l_Caption,l_pos,1);
  // check if item have submenues
  if TMenuItem(Sender).Count > 0 then
        l_submenuPlaceholder := m_submenuArrowPlaceHolder;
  // Relativ Width looks mutch nicer
  if m_relativWidth then begin
        // get to used font
        ACanvas.Font := m_Menu_Font.GetFont;
        ACanvas.Font.Size := m_Menu_Font.GetFont.Size;
        if TMenuItem(Sender).RadioItem or TMenuItem(Sender).AutoCheck then begin
                l_iconWidth            := m_itemsheight;
        end;
        if (TMenuItem(Sender).HelpContext = LCSUBMENUGROUP) or
           (TMenuItem(Sender).HelpContext = LCICONSUBMENUGROUP) then begin
                l_submenuPlaceholder := m_submenuArrowPlaceHolder;
        end;

        if m_iconsOn then begin
                if not (TMenuItem(Sender).Hint = '') then begin
                        l_iconRightPlaceholder := m_iconRightPlaceHolder;
                        l_iconWidth            := m_itemsheight;
                end;
                if (TMenuItem(Sender).HelpContext = LCICONGROUP) then begin
                        l_iconRightPlaceholder := m_iconRightPlaceHolder;
                        l_iconWidth            := m_itemsheight;
                end;
                if (TMenuItem(Sender).HelpContext = LCICONSUBMENUGROUP) then begin
                        l_iconRightPlaceholder := m_iconRightPlaceHolder;
                        l_iconWidth            := m_itemsheight;
                        l_submenuPlaceholder   := m_submenuArrowPlaceHolder;
                end;
        end;
        width :=        m_leftOffset + m_TextLeft +
                        ACanvas.TextWidth(l_Caption) +
                        m_TextRight + l_submenuPlaceholder +
                        l_iconRightPlaceholder +  l_iconWidth -
                        lc_MeasureDrawItemInconsistenz;
  end else
        width := m_itemswidth;
  // blank command have another height
  if TMenuItem(Sender).Caption = '-' then
        height := m_blankHeight
  else
        height := m_itemsheight;
end;

procedure TdspPopupMenu.OnAdvanDrawItem(Sender: TObject; ACanvas: TCanvas; ARect: TRect; State: TOwnerDrawState);
var
  l_Caption: string;
  l_pos: integer;
  l_itemSelected: boolean;
  l_Font: TFont;
  l_msg: TMSG;
  l_Rect: TRect;
  l_MenuItem: TMenuItem;
  l_icon: Graphics.TBitmap;
  l_finalIcon: Graphics.TBitmap;
  l_tempColor: TColor;
  l_iconfound: boolean;
  l_iconRightPlaceHolder: integer;
  l_iconWidth: integer;
  l_textWidth: integer;
  l_textHeight: integer;
  l_i: integer;
  l_submenu: boolean;
  hWndM: HWND;
begin

 // g_errLog.AddErrStr('TdspPopupMenu.OnAdvanDrawItem : ' + TMenuItem(Sender).Name);
  l_iconRightPlaceHolder := 0;
  l_iconWidth            := 0;
  l_MenuItem := TMenuItem(Sender);
  l_submenu := false;
  if l_MenuItem.Count > 0 then
        l_submenu := true;
  l_iconfound := false;

  // get to used font
  if not (ACanvas.Font.Name = m_Menu_Font.GetFont.Name) then
        ACanvas.Font := m_Menu_Font.GetFont;
  l_itemSelected := false;
  // get the caption and its width
  l_Caption := TMenuItem(sender).Caption;
  l_pos := Pos('&',l_Caption);
  if l_pos > 0 then
        Delete(l_Caption,l_pos,1);
  l_textWidth := ACanvas.TextWidth(l_Caption);
  l_textHeight := ACanvas.TextHeight(l_Caption);
  // default drawing
  ACanvas.Brush.Color := m_Backcolor;
  ACanvas.Font.Color := m_fontColor;
  //ACanvas.FillRect(ARect);
  // Item is selected
  if odSelected in State then begin
       // selected drawing settings
       ACanvas.Brush.Color := m_SelectedBackcolor;
       ACanvas.Font.Color := m_SelectedFontColor;
       l_itemSelected := true;
       if icmLineIncluded in m_iconMode then begin
                m_iconTempForeColor := SelectedFontColor;
                m_iconTempBackcolor := SelectedBackColor;
       end;
       if icmSeperateColors in m_iconMode then begin
                m_iconTempForeColor := m_iconSelectedForeColor;
                m_iconTempBackcolor := m_iconSelectedBackcolor;
       end;
  end else begin
       if icmLineIncluded in m_iconMode then begin
                m_iconTempForeColor := FontColor;
                m_iconTempBackcolor := Backcolor;
       end;
       if icmSeperateColors in m_iconMode then begin
                m_iconTempForeColor := m_iconForeColor;
                m_iconTempBackcolor := m_iconBackcolor;
       end;
  end;
  if (m_iconMode = []) or (m_iconMode = [icm3DBorder])  then begin
        m_iconTempForeColor := FontColor;
        m_iconTempBackcolor := Backcolor;
  end;
  if l_itemSelected then begin
        if odDisabled in State then begin
                m_iconTempForeColor := m_disabledSelectedColor;
                m_iconTempBackcolor := m_SelectedBackcolor;
        end;
  end else begin
        if odDisabled in State then begin
                m_iconTempForeColor := m_disabledColor;
                m_iconTempBackcolor := m_Backcolor;
        end;
  end;

  // Item is a radioItem
  if TMenuItem(Sender).RadioItem or TMenuItem(Sender).AutoCheck then begin
        l_iconWidth            := m_itemsheight;
  end;
  // Icon support enabled
  if m_iconsOn then begin
        // Item is an iconItem
        if not (TMenuItem(Sender).Hint = '') then begin
                l_iconRightPlaceholder := m_iconRightPlaceHolder;
                l_iconWidth            := m_itemsheight;
        end;
        if (TMenuItem(Sender).HelpContext = LCICONGROUP) or
           (TMenuItem(Sender).HelpContext = LCICONSUBMENUGROUP) then begin
                l_iconRightPlaceholder := m_iconRightPlaceHolder;
                l_iconWidth            := m_itemsheight;
        end;
  end;
  //set the Font Color
  if l_itemSelected then begin
        if odDisabled in State then
                ACanvas.Font.Color := m_disabledSelectedColor;
  end else begin
        if odDisabled in State then
                ACanvas.Font.Color := m_disabledColor;
  end;
  // draw the TMenuitem.caption to canvas
  if l_Caption <> '-' then
        ACanvas.TextOut(m_leftOffset + l_iconWidth + l_iconRightPlaceHolder + m_TextLeft,ARect.Top+(trunc((m_itemsheight-ACanvas.TextHeight(l_Caption))/2)),l_Caption);
  // draw the "  -  " command
  if TMenuItem(Sender).Caption = '-' then begin
        m_blankIndex := m_blankIndex+1;
        ACanvas.Brush.Color := m_Backcolor;
        ACanvas.Pen.Color := m_BlankColor;
        ARect.Bottom := ARect.Top+m_blankHeight;
        ACanvas.FillRect(ARect);
        ACanvas.MoveTo(3,ARect.Bottom-((ARect.Bottom-ARect.Top) div 2));
        ACanvas.LineTo(ARect.Right-3,ARect.Bottom-((ARect.Bottom-ARect.Top) div 2));
        Exit;
  end;
  // set the colors for left background
  l_tempColor := ACanvas.Brush.Color;
  if m_iconsOn then begin
        if (icmLineIncluded in m_iconMode) then
                if l_itemSelected then
                        ACanvas.Brush.Color := m_SelectedBackcolor
                else
                        ACanvas.Brush.Color := m_Backcolor
        else begin
                if not (l_MenuItem.Hint = '') then begin
                        ACanvas.Brush.Color := m_Backcolor;
                        if l_itemSelected then begin
                                if odDisabled in State then
                                        ACanvas.Brush.Color := m_SelectedBackcolor;
                        end else begin
                                        if odDisabled in State then
                                                ACanvas.Brush.Color := m_Backcolor;
                        end;
                end else begin
                        if l_itemSelected then
                                ACanvas.Brush.Color := m_SelectedBackcolor
                        else
                                ACanvas.Brush.Color := m_Backcolor;
                end;
        end;
  end;
  // paint the left-left background
  l_Rect.Top := ARect.Top;
  l_Rect.Left := ARect.Left;
  l_Rect.Right := ARect.Left+m_leftOffset;
  l_Rect.Bottom := ARect.Bottom;
  ACanvas.FillRect(l_Rect);
  ACanvas.Brush.Color := l_tempColor;
  // paint the left-right background
  l_Rect.Top := ARect.Top;
  l_Rect.Left := ARect.Left+m_leftOffset+l_iconWidth+l_iconRightPlaceHolder;
  l_Rect.Right := ARect.Left+m_leftOffset+l_iconWidth+l_iconRightPlaceHolder+m_TextLeft;
  l_Rect.Bottom := ARect.Bottom;
  ACanvas.FillRect(l_Rect);
  // paint the right background
  l_Rect.Top := ARect.Top;
  l_Rect.Left := ARect.Left+m_leftOffset+l_iconWidth+l_iconRightPlaceHolder+m_TextLeft+l_textWidth;
  l_Rect.Right := ARect.Right;
  l_Rect.Bottom := ARect.Bottom;
  ACanvas.FillRect(l_Rect);
  // paint the radioItem Background
  if l_MenuItem.RadioItem or TMenuItem(Sender).AutoCheck then begin
        l_Rect.Top := ARect.Top;
        l_Rect.Left := ARect.Left+m_leftOffset;
        l_Rect.Right := ARect.Left+m_leftOffset+l_iconWidth+l_iconRightPlaceHolder;
        l_Rect.Bottom := ARect.Bottom;
        ACanvas.FillRect(l_Rect);
  end;
  // paint needed rects if " textheight < Itemheight "
  if l_textHeight < (ARect.Bottom-ARect.Top) then begin
        l_Rect.Top := ARect.Top;
        l_Rect.Bottom := ARect.Top+(trunc((m_itemsheight-ACanvas.TextHeight(l_Caption))/2));
        l_Rect.Left := m_leftOffset + l_iconWidth + l_iconRightPlaceHolder + m_TextLeft;
        l_Rect.Right := m_leftOffset + l_iconWidth + l_iconRightPlaceHolder + m_TextLeft + l_textWidth;
        ACanvas.FillRect(l_Rect);
        l_Rect.Top := ARect.Top+(trunc((m_itemsheight-ACanvas.TextHeight(l_Caption))/2)) + l_textHeight;
        l_Rect.Bottom := ARect.Bottom;
        l_Rect.Left := m_leftOffset + l_iconWidth + l_iconRightPlaceHolder + m_TextLeft;
        l_Rect.Right := m_leftOffset + l_iconWidth + l_iconRightPlaceHolder + m_TextLeft + l_textWidth;
        ACanvas.FillRect(l_Rect);
  end;
  if m_iconsOn then begin
     // Draw the Icon
     if not (l_MenuItem.Hint = '') then begin
        // check if file exist
        if (FileExists(m_IconPath + '\' + l_MenuItem.Hint)) then begin
                l_iconfound := true;
                l_finalIcon := Graphics.TBitmap.Create;
                l_icon := Graphics.TBitmap.Create;
                l_tempColor := ACanvas.Brush.Color;
                if (icmLineIncluded in m_iconMode) then
                        if l_itemSelected then
                                ACanvas.Brush.Color := m_SelectedBackcolor
                        else
                        ACanvas.Brush.Color := m_Backcolor
                else
                        ACanvas.Brush.Color := m_Backcolor;
                if l_itemSelected then begin
                        if odDisabled in State then
                                ACanvas.Brush.Color := m_SelectedBackcolor;
                end else begin
                                if odDisabled in State then
                                        ACanvas.Brush.Color := m_Backcolor;
                end;
                // draw the right Icon offset
                l_Rect.Top := ARect.Top;
                l_Rect.Bottom := ARect.Bottom;
                l_Rect.Left := m_leftOffset+l_iconWidth;
                l_Rect.Right := ARect.Left+m_leftOffset+l_iconWidth+l_iconRightPlaceHolder;
                ACanvas.FillRect(l_Rect);
                ACanvas.Brush.Color := l_tempColor;
                // draw the Icon
                l_icon.LoadFromFile(m_IconPath + '\' + l_MenuItem.Hint);
                getIconImage(l_icon,l_finalIcon);
                l_Rect.Top := ARect.Top;
                l_Rect.Bottom := ARect.Bottom;
                l_Rect.Left := ARect.Left+m_leftOffset;
                l_Rect.Right := l_iconWidth+m_leftOffset;
                ACanvas.StretchDraw(l_Rect,l_finalIcon);
                // Draw the 3d border
                if (icm3DBorder in m_iconMode) and not (odDisabled in State) then begin
                    l_tempColor := ACanvas.Pen.Color;
                    ACanvas.Pen.Width := 1;
                    if l_itemSelected then
                        ACanvas.Pen.Color := m_icon3dLeftSelectedTopEdgeBordersColor
                    else
                        ACanvas.Pen.Color := m_icon3dLeftTopEdgeBordersColor;
                    ACanvas.MoveTo(ARect.Left+m_leftOffset,ARect.Top);
                    ACanvas.LineTo(l_iconWidth+m_leftOffset,ARect.Top);
                    ACanvas.MoveTo(ARect.Left+m_leftOffset,ARect.Top);
                    ACanvas.LineTo(ARect.Left+m_leftOffset,ARect.Bottom);
                    if l_itemSelected then
                        ACanvas.Pen.Color := m_icon3dRightSelectedBottomEdgeBordersColor
                    else
                        ACanvas.Pen.Color := m_icon3dRightBottomEdgeBordersColor;
                    ACanvas.MoveTo(ARect.Bottom-ARect.Top+m_leftOffset-1,ARect.Bottom-1);
                    ACanvas.LineTo(ARect.Left+m_leftOffset-1,ARect.Bottom-1);
                    ACanvas.MoveTo(ARect.Bottom-ARect.Top+m_leftOffset-1,ARect.Bottom-1);
                    ACanvas.LineTo(ARect.Bottom-ARect.Top+m_leftOffset-1,ARect.Top-1);
                    ACanvas.Pen.Color := l_tempColor;
                end;
                l_icon.Destroy;
                l_finalIcon.Destroy;
        end else begin
                // File do not exist
                if not (l_MenuItem.Hint = '') then
                        g_errLog.AddErrStr('TdspPopupMenu.OnAdvanDrawItem:  File: ' +
                                   m_IconPath + '\' + l_MenuItem.Hint + 'do not esist');
        end;
     end else begin
                if (TMenuItem(Sender).HelpContext = LCICONGROUP) or
                   (TMenuItem(Sender).HelpContext = LCICONSUBMENUGROUP) then begin
                                if l_itemSelected then
                                        ACanvas.Brush.Color := m_SelectedBackcolor
                                else
                                ACanvas.Brush.Color := m_Backcolor;
                        l_Rect.Top := ARect.Top;
                        l_Rect.Bottom := ARect.Bottom;
                        l_Rect.Left := ARect.Left+m_leftOffset;
                        l_Rect.Right := l_iconWidth+m_leftOffset;
                        ACanvas.FillRect(l_Rect);
                        // draw the right Icon offset
                        l_Rect.Top := ARect.Top;
                        l_Rect.Bottom := ARect.Bottom;
                        l_Rect.Left := m_leftOffset+l_iconWidth;
                        l_Rect.Right := ARect.Left+m_leftOffset+l_iconWidth+l_iconRightPlaceHolder;
                        ACanvas.FillRect(l_Rect);
                end;
     end;
   end;

  // draw the Checked Symbol
  if odChecked in State then begin
        if l_itemSelected then
                ACanvas.Pen.Color := m_selectedCheckedColor
        else
                ACanvas.Pen.Color := m_CheckedColor;
        ACanvas.Pen.Width := 2;
        ACanvas.MoveTo(3+m_leftOffset,ARect.Bottom-((ARect.Bottom-ARect.Top) div 2));
        ACanvas.LineTo(((ARect.Bottom-ARect.Top) div 3)-1+m_leftOffset,ARect.Bottom-((ARect.Bottom-ARect.Top) div 4)-1);
        ACanvas.LineTo(ARect.Bottom-ARect.Top-((ARect.Bottom-ARect.Top) div 4)-1+m_leftOffset,ARect.Top+((ARect.Bottom-ARect.Top) div 4)+1);
  end;

  //for l_i := 0 to 10 do begin
  hWndM := WindowFromDC(ACanvas.Handle);
  if (hWndM<>0) then begin
     DrawWindowBorder(hWndM);
  end else
     if not PeekMessage(l_msg,Application.Handle,WM_DRAWMENUBORDER,WM_DRAWMENUBORDER,PM_NOREMOVE) then
        PostMessage(Application.Handle,WM_DRAWMENUBORDER,0,Integer(TMenuItem(Sender)));


  if m_submenuSkinningEnabled then

        if l_submenu then

           if not (odDisabled in State) then begin

                if l_itemSelected then begin

                        // Draw the arrow Field
                     if not PeekMessage(l_msg,Application.Handle,WM_ARROW,LCSELECTED,WM_ARROW) then
                        PostMessage(Application.Handle,WM_ARROW,LCSELECTED,Integer(TMenuItem(Sender)));
                end else begin
                  if not PeekMessage(l_msg,Application.Handle,WM_ARROW,0,WM_ARROW) then
                        PostMessage(Application.Handle,WM_ARROW,0,Integer(TMenuItem(Sender)));
                end;
            end else begin
                if l_itemSelected then begin
                        // Draw the arrow Field
                      if not PeekMessage(l_msg,Application.Handle,WM_ARROW,LCDISABLEDSELECTED,WM_ARROW) then
                        PostMessage(Application.Handle,WM_ARROW,LCDISABLEDSELECTED,Integer(TMenuItem(Sender)));
                end else begin
                 if not PeekMessage(l_msg,Application.Handle,WM_ARROW,LCDISABLED,WM_ARROW) then
                        PostMessage(Application.Handle,WM_ARROW,LCDISABLED,Integer(TMenuItem(Sender)));
                end;
            end;
end;

procedure TdspPopupMenu.OnPopup(Sender: TObject);
begin
 // WORKAROUND   -  strange iusse in baseclass TPopupmenu  -
 // OnMeasureItem in the submenus is only called on time after reset
 // m_popupmenu.OwnerDraw := true;
 // so we reset the Ownerdraw on every m_popupmenu popupevent
 if m_skinningEnabled then begin
        m_popupmenu.OwnerDraw := false;
        m_popupmenu.OwnerDraw := true;
 end;
 // send the event to Receiver if assigned
 if assigned(m_onMenuPopup) then
        m_onMenuPopup(self);
end;

end.
 