unit dspPopupMenuGroup;

    (*********************************************************************
     *  unit dspPopupMenuGroup                                           *
     *                                                                   *
     *  home page : http://www.dsplayer.de                               *
     *  email     : coder@dsplayer.de                                    *
     *                                                                   *
     *                                                                   *
     *  date      : 15-03-2004                                           *
     *                                                                   *
     *  The contents of this file are used with permission, subject to   *
     *  the Mozilla Public License Version 1.1 (the "License"); you may  *
     *  not use this file except in compliance with the License. You may *
     *  obtain a copy of the License at                                  *
     *  http://www.mozilla.org/MPL/MPL-1.1.html                          *
     *                                                                   *
     *  Software distributed under the License is distributed on an      *
     *  "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   *
     *  implied. See the License for the specific language governing     *
     *  rights and limitations under the License.                        *
     *                                                                   *
     *********************************************************************)

{
  @abstract(DSPlayer Components.)
  @author(Martin Offenwanger: coder@dsplayer.de)
  @created(Mar 15, 2004)
  @lastmod(Apr 19, 2004)
}

// DSPlayer Popup Manager Class
// Handles all used Menues

interface

uses
   Menus, Graphics, Classes, SysUtils, dspPopupMenu, dspPopupDataTypes,
   dspLanguage;
type
   TPopupMenuGroup = class(TComponent)
   public
     constructor Create(AOwner: TComponent);
     destructor Destroy;
     function getFontFile: string;
     function getFontSize: integer;
     function getFontStyles: string;
     function getFontName: String;
     procedure SaveCurrentSettings(f_file: string);
     procedure loadCurrentSettingsFromDsfFile(f_file: string);
     function addPopupmenuByDefFile(f_deffile: string; f_name: string): boolean;
     procedure setEntryToUseAsDynamicSubmenuRoot(f_SubmenuRootName: string);
     procedure setUncheckedItemsOnPopup(f_uncheckedItemsOnPopup: string);
     function getPopupmenuByNameAsTpopupmenu(f_Name: string): TPopupmenu;
     function getPopupmenuByName(f_Name: string): TdspPopupmenu;
     Procedure setIconPath(f_Iconpath: string);
     procedure setLanguageObject(var f_languageObject: TdspLanguage);
     procedure setFontfilePath(f_path: string);
     procedure setFont(f_fontFile: string; f_fontSize: integer; f_fontstyles: string = '');
   private
     m_language: TdspLanguage;
     m_languageCommands: TStrings;
     m_rootsOfDynamicSubmenues: TStrings;
     m_uncheckedItemsOnPopup: TStrings;
     m_blankColor: TColor;
     m_backcolor: TColor;
     m_selectedBackcolor: TColor;
     m_fontColor: TColor;
     m_selectedFontColor: TColor;
     m_checkedColor: TColor;
     m_selectedCheckedColor: TColor;
     m_borderColor: TColor;
     m_TextLeft: integer;
     m_TextRight: integer;
     m_blankHeight: integer;
     m_itemsheight: integer;
     m_itemswidth: integer;
     m_relativWidth: boolean;
     m_leftOffset:  integer;
     m_submenuArrowPlaceHolder: integer;
     m_SubmenuArrowColor: TColor;
     m_SubmenuArrowSelectedColor: TColor;
     m_iconRightPlaceHolder: integer;
     m_iconBackColor: TColor;
     m_iconForeColor: TColor;
     m_iconSelectedBackColor: TColor;
     m_iconSelectedForeColor: TColor;
     m_icon3dRightBottomEdgeBordersColor : TColor;
     m_icon3dLeftTopEdgeBordersColor : TColor;
     m_icon3dRightSelectedBottomEdgeBordersColor : TColor;
     m_icon3dLeftSelectedTopEdgeBordersColor : TColor;
     m_iconMode: TIconMode;
     m_iconsOn: boolean;
     m_disabledColor: TColor;
     m_disabledSelectedColor: TColor;
     m_skinningEnabled: boolean;
     m_submenuSkinningEnabled: boolean;
     m_fontfilepath: string;
     m_popupmenus: array of TdspPopupMenu;
     m_onMenuItemClick: TOnMenuItemClick;
     m_onSubmenuDelete: TOnMenuSubmenuDelete;
     m_onUncheckItem: TOnMenuUncheckItem;
     m_onMenuPopup: TOnMenuPopup;
     // Files and Folders
     procedure setSubmenuSkinningEnabled(f_submenuSkinningEnabled: boolean);
     procedure setSubmenuArrowColor(f_submenuArrowColor: TColor);
     procedure setSubmenuArrowSelectedColor(f_submenuArrowSelectedColor: TColor);
     procedure setSkinningEnabled(f_skinningEnabled: boolean);
     procedure setDisabledColor(f_disabledColor: TColor);
     procedure setDisabledSelectedColor(f_disabledSelectedColor: TColor);
     procedure setIconsOn(f_IconsOn: boolean);
     procedure setLeftOffset(f_leftOffset: Integer);
     procedure setSubmenuArrowPlaceHolder(f_submenuArrowPlaceHolder: integer);
     procedure SetIconRightPlaceHolder(f_iconRightPlaceHolder: Integer);
     procedure setIconBackColor(f_iconBackColor: TColor);
     procedure setIconForeColor(f_iconForeColor: TColor);
     procedure setIconSelectedBackColor(f_iconSelectedBackColor: TColor);
     procedure setIconSelectedForeColor(f_iconSelectedForeColor: TColor);
     procedure SetIcon3dRightBottomEdgeBordersColor(f_icon3dRightBottomEdgeBordersColor: TColor);
     procedure SetIcon3dLeftTopEdgeBordersColor(f_icon3dLeftTopEdgeBordersColor: TColor);
     procedure SetIcon3dRightSelectedBottomEdgeBordersColor(f_icon3dRightSelectedBottomEdgeBordersColor: TColor);
     procedure SetIcon3dLeftSelectedTopEdgeBordersColor(f_icon3dLeftSelectedTopEdgeBordersColor: TColor);
     procedure setIconMode(f_IconMode: TIconMode);
     procedure setBlankColor(f_blankColor: TColor);
     procedure setBackColor(f_backcolor: TColor);
     procedure setSelectedBackColor(f_SelectedBackcolor: TColor);
     procedure setFontColor(f_fontcolor: TColor);
     procedure setSelectedFontColor(f_selectedFontColor: TColor);
     procedure setCheckedColorColor(f_checkedColor: TColor);
     procedure setSelectedCheckedColor(f_selectedCheckedColor: TColor);
     procedure setBorderColor(f_BorderColor: TColor);
     procedure setTextLeft(f_textLeft: integer);
     procedure setTextRight(f_textRight: integer);
     procedure setBlankHeight(f_blankHeight: integer);
     procedure setItemsHeight(f_itemsHeight: integer);
     procedure setItemsWidth(f_itemsWidth: integer);
     procedure setRelativWidth(f_relativWidth: boolean);
     procedure OnDspPopupMenuItemClick(Sender: TObject; f_ItemName: string);
     procedure OnMenuClientMenuPopup(Sender: TObject);
     function IsLanguageFileCommand(f_ItemName: string): boolean;
     function SetPopupmenusLanguageCaptions(f_languageDescription: string): boolean;
     procedure RefreshCompleteSkinningConfig;
   published
     property BlankColor: TColor read m_BlankColor write setBlankColor;
     property Backcolor: TColor read m_Backcolor write setBackColor;
     property SelectedBackColor: TColor read m_SelectedBackcolor write setSelectedBackColor;
     property FontColor: TColor read m_fontColor write setFontColor;
     property SelectedFontColor: TColor read m_selectedFontColor write setSelectedFontColor;
     property CheckedColor: TColor read m_checkedColor write setCheckedColorColor;
     property SelectedCheckedColor: TColor read m_selectedCheckedColor write setSelectedCheckedColor;
     property BorderColor: TColor read m_borderColor write setBorderColor;
     property TextLeft: integer read m_TextLeft write setTextLeft;
     property TextRight: integer read m_TextRight write setTextRight;
     property BlankHeigt: integer read m_blankHeight write setBlankHeight;
     property ItemsHeight: integer read m_itemsheight write setItemsHeight;
     property ItemsWidth: integer read m_itemswidth write setItemsWidth;
     property RelativWidth: boolean read m_relativWidth write setRelativWidth;
     property IconRightPlaceHolder: Integer read m_iconRightPlaceHolder write SetIconRightPlaceHolder;
     property IconBackColor: TColor read m_iconBackColor write setIconBackColor;
     property IconForeColor: TColor read m_iconForeColor write SetIconForeColor;
     property IconSelectedBackColor: TColor read m_iconSelectedBackColor write setIconSelectedBackColor;
     property IconSelectedForeColor: TColor read m_iconSelectedForeColor write setIconSelectedForeColor;
     property Icon3dRightBottomEdgeBordersColor: TColor read m_icon3dRightBottomEdgeBordersColor write SetIcon3dRightBottomEdgeBordersColor;
     property Icon3dLeftTopEdgeBordersColor: TColor read m_icon3dLeftTopEdgeBordersColor write SetIcon3dLeftTopEdgeBordersColor;
     property Icon3dRightSelectedBottomEdgeBordersColor: TColor read m_icon3dRightSelectedBottomEdgeBordersColor write SetIcon3dRightSelectedBottomEdgeBordersColor;
     property Icon3dLeftSelectedTopEdgeBordersColor: TColor read m_icon3dLeftSelectedTopEdgeBordersColor write SetIcon3dLeftSelectedTopEdgeBordersColor;
     property DisabledColor: TColor read m_disabledColor write setDisabledColor;
     property DisabledSelectedColor: TColor read m_disabledSelectedColor write setDisabledSelectedColor;
     property IconMode: TIconMode read m_iconMode write setIconMode;
     property IconsOn: Boolean read m_iconsOn write setIconsOn;
     property LeftOffset: Integer read m_leftOffset write setLeftOffset;
     property SubmenuArrowPlaceHolder: Integer read m_submenuArrowPlaceHolder write setSubmenuArrowPlaceHolder;
     property SubmenuArrowColor: TColor read m_SubmenuArrowColor write setSubmenuArrowColor;
     property SubmenuArrowSelectedColor: TColor read m_SubmenuArrowSelectedColor write setSubmenuArrowSelectedColor;
     property SkinningEnabled: boolean read m_skinningEnabled write setSkinningEnabled;
     // Files and Folders
    // property ConfigFilePath: string read m_configFilePath write m_configFilePath;
     property SubmenuSkinningEnabled: boolean read m_submenuSkinningEnabled write setSubmenuSkinningEnabled;
     // Events
     property OnMenuItemClick: TOnMenuItemClick read m_onMenuItemClick write m_onMenuItemClick;
     property OnSubmenuDelete: TOnMenuSubmenuDelete read m_onSubmenuDelete write m_onSubmenuDelete;
     property OnMenuUncheckItem: TOnMenuUncheckItem read m_onUncheckItem write m_onUncheckItem;
     property OnPopup: TOnMenuPopup read m_onMenuPopup write m_onMenuPopup;
end;

implementation

uses Globals;

procedure TPopupMenuGroup.setEntryToUseAsDynamicSubmenuRoot(f_SubmenuRootName: string);
begin
  m_rootsOfDynamicSubmenues.Add(f_SubmenuRootName);
end;

procedure TPopupMenuGroup.setUncheckedItemsOnPopup(f_uncheckedItemsOnPopup: string);
begin
  m_uncheckedItemsOnPopup.Add(f_uncheckedItemsOnPopup);
end;

procedure TPopupMenuGroup.setLanguageObject(var f_languageObject: TdspLanguage);
begin
  m_language := f_languageObject;
  SetPopupmenusLanguageCaptions(m_language.getActiveLanguageAsDescription);

end;

Procedure TPopupMenuGroup.setIconPath(f_Iconpath: string);
var
  l_i: integer;
begin
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].IconPath := f_Iconpath;
end;

function TPopupMenuGroup.getPopupmenuByNameAsTpopupmenu(f_Name: string): TPopupmenu;
var
  l_i: integer;
begin
  RESULT := nil;
  for l_i := 0 to high(m_popupmenus) do
        if m_popupmenus[l_i].Name = f_Name then begin
                RESULT := m_popupmenus[l_i].getAsTPopupMenu;
                break;
        end;
end;

function TPopupMenuGroup.getPopupmenuByName(f_Name: string): TdspPopupmenu;
var
  l_i: integer;
begin
  RESULT := nil;
  for l_i := 0 to high(m_popupmenus) do
        if m_popupmenus[l_i].Name = f_Name then begin
                RESULT := m_popupmenus[l_i];
                break;
        end;
end;

function TPopupMenuGroup.addPopupmenuByDefFile(f_deffile: string; f_name: string): boolean;
var
  l_temp: TStrings;
begin
  l_temp := TStringList.Create;
  setlength(m_popupmenus,high(m_popupmenus)+2);
  m_popupmenus[high(m_popupmenus)] := TdspPopupMenu.Create(self);
  m_popupmenus[high(m_popupmenus)].Name := f_name;
  m_popupmenus[high(m_popupmenus)].generateMenueEntriesByDefFile(f_deffile,l_temp);
  m_popupmenus[high(m_popupmenus)].OnMenuItemClick := OnDspPopupMenuItemClick;
  m_popupmenus[high(m_popupmenus)].OnMenuPopup := OnMenuClientMenuPopup;
  if not (m_language = nil) then begin
        if not m_language.setLanguageFileByDecription(m_language.getActiveLanguageAsDescription) then begin
                g_errLog.AddErrStr('TPopupMenuGroup.SetPopupmenusLanguageCaptions: SetLanguageFile by Description failed');
                Result := false;
                exit;
        end;
        // get and set the setuppopupmenumenu ItemCaptions
        if not m_language.getPopupMenuCaptions(l_temp, m_popupmenus[high(m_popupmenus)].getDefFileMenuItemNames) then begin
                g_errLog.AddErrStr('TPopupMenuGroup.SetPopupmenusCaptions: get SetupMenucaptions failed');
                Result := false;
        end else
                m_popupmenus[high(m_popupmenus)].setMenuEntryCaptions(l_temp);
  end;
  RefreshCompleteSkinningConfig;
  l_temp.Destroy;    
end;


procedure TPopupMenuGroup.RefreshCompleteSkinningConfig;
begin

  if high(m_popupmenus) < 0 then begin
        g_errLog.AddErrStr('TPopupMenuGroup.RefreshCompleteSkinningConfig ' +
                           'error: can not refresh, no popupmenues are existing');
        exit;
  end;
  setBorderColor(m_popupmenus[0].BorderColor);
  setBlankColor(m_popupmenus[0].BlankColor);
  setBackColor(m_popupmenus[0].Backcolor);
  setSelectedBackColor(m_popupmenus[0].SelectedBackColor);
  setFontColor(m_popupmenus[0].FontColor);
  setSelectedFontColor(m_popupmenus[0].SelectedFontColor);
  setCheckedColorColor(m_popupmenus[0].CheckedColor);
  setSelectedCheckedColor(m_popupmenus[0].SelectedCheckedColor);
  setTextLeft(m_popupmenus[0].TextLeft);
  setTextRight(m_popupmenus[0].TextRight);
  setBlankHeight(m_popupmenus[0].BlankHeigt);
  setItemsHeight(m_popupmenus[0].ItemsHeight);
  setItemsWidth(m_popupmenus[0].ItemsWidth);
  setRelativWidth(m_popupmenus[0].RelativWidth);
  setIconBackColor(m_popupmenus[0].IconBackColor);
  setIconForeColor(m_popupmenus[0].IconForeColor);
  setIconSelectedBackColor(m_popupmenus[0].IconSelectedBackColor);
  setIconSelectedForeColor(m_popupmenus[0].IconSelectedForeColor);
  setLeftOffset(m_popupmenus[0].leftOffset);
  SetIconRightPlaceHolder(m_popupmenus[0].IconRightPlaceHolder);
  setSubmenuArrowPlaceHolder(m_popupmenus[0].SubmenuArrowPlaceHolder);
  setIconsOn(m_popupmenus[0].IconsOn);
  SetIcon3dRightBottomEdgeBordersColor(m_popupmenus[0].Icon3dRightBottomEdgeBordersColor);
  SetIcon3dLeftTopEdgeBordersColor(m_popupmenus[0].Icon3dLeftTopEdgeBordersColor);
  SetIcon3dRightSelectedBottomEdgeBordersColor(m_popupmenus[0].Icon3dRightSelectedBottomEdgeBordersColor);
  SetIcon3dLeftSelectedTopEdgeBordersColor(m_popupmenus[0].Icon3dLeftSelectedTopEdgeBordersColor);
  setDisabledColor(m_popupmenus[0].DisabledColor);
  setDisabledSelectedColor(m_popupmenus[0].DisabledSelectedColor);
  setIconMode(m_popupmenus[0].IconMode);
  setSubmenuArrowColor(m_popupmenus[0].SubmenuArrowColor);
  setSubmenuArrowSelectedColor(m_popupmenus[0].SubmenuArrowSelectedColor);
  setSkinningEnabled(m_popupmenus[0].SkinningEnabled);
  setSubmenuSkinningEnabled(m_popupmenus[0].SubmenuSkinningEnabled);
end;

procedure TPopupMenuGroup.loadCurrentSettingsFromDsfFile(f_file: string);
var
  l_i: integer;
begin
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].loadCurrentSettingsFromDsfFile(f_file);
  RefreshCompleteSkinningConfig;
end;

procedure TPopupMenuGroup.SaveCurrentSettings(f_file: string);
begin
  m_popupmenus[0].saveCurrentSettingsToDsfFile(f_file);
end;

function TPopupMenuGroup.getFontFile: string;
begin
 Result := m_popupmenus[0].getFontFile;;
end;

function TPopupMenuGroup.getFontSize: integer;
begin
 Result := m_popupmenus[0].getFontSize;;
end;

function TPopupMenuGroup.getFontStyles: string;
begin
 Result := m_popupmenus[0].getFontStyles;;
end;

function TPopupMenuGroup.getFontName: String;
begin
 Result := m_popupmenus[0].getFontName;
end;

procedure TPopupMenuGroup.setSubmenuSkinningEnabled(f_submenuSkinningEnabled: boolean);
var
  l_i: integer;
begin
  m_submenuSkinningEnabled := f_submenuSkinningEnabled;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].SubmenuSkinningEnabled := f_submenuSkinningEnabled;
end;

procedure TPopupMenuGroup.setSubmenuArrowColor(f_submenuArrowColor: TColor);
var
  l_i: integer;
begin
  m_submenuArrowColor := f_submenuArrowColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].SubmenuArrowColor := f_submenuArrowColor;
end;

procedure TPopupMenuGroup.setSubmenuArrowSelectedColor(f_submenuArrowSelectedColor: TColor);
var
  l_i: integer;
begin
  m_submenuArrowSelectedColor := f_submenuArrowSelectedColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].SubmenuArrowSelectedColor := f_submenuArrowSelectedColor;
end;

procedure TPopupMenuGroup.setSkinningEnabled(f_skinningEnabled: boolean);
var
  l_i: integer;
begin
  m_skinningEnabled := f_skinningEnabled;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].SkinningEnabled := f_skinningEnabled;
end;

procedure TPopupMenuGroup.setIconsOn(f_IconsOn: boolean);
var
  l_i: integer;
begin
  m_IconsOn := f_IconsOn;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].IconsOn := f_IconsOn;
end;

procedure TPopupMenuGroup.setIconMode(f_IconMode: TIconMode);
var
  l_i: integer;
begin
  m_IconMode := f_IconMode;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].IconMode := f_IconMode;
end;

procedure TPopupMenuGroup.setSubmenuArrowPlaceHolder(f_submenuArrowPlaceHolder: integer);
var
  l_i: integer;
begin
  m_submenuArrowPlaceHolder := f_submenuArrowPlaceHolder;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].SubmenuArrowPlaceHolder := f_submenuArrowPlaceHolder;
end;

procedure TPopupMenuGroup.setLeftOffset(f_leftOffset: Integer);
var
  l_i: integer;
begin
  m_leftOffset := f_leftOffset;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].leftOffset := f_leftOffset;
end;


procedure TPopupMenuGroup.setIconRightPlaceHolder(f_iconRightPlaceHolder: Integer);
var
  l_i: integer;
begin
  m_iconRightPlaceHolder := f_iconRightPlaceHolder;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].IconRightPlaceHolder := f_iconRightPlaceHolder;
end;


procedure TPopupMenuGroup.setIconBackColor(f_iconBackColor: TColor);
var
  l_i: integer;
begin
  m_iconBackColor := f_iconBackColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].IconBackColor := f_iconBackColor;
end;

procedure TPopupMenuGroup.setIconForeColor(f_iconForeColor: TColor);
var
  l_i: integer;
begin
  m_iconForeColor := f_iconForeColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].IconForeColor := f_iconForeColor;
end;

procedure TPopupMenuGroup.setIconSelectedBackcolor(f_iconSelectedBackColor: TColor);
var
  l_i: integer;
begin
  m_iconSelectedBackColor := f_iconSelectedBackColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].IconSelectedBackColor := f_iconSelectedBackColor;
end;

procedure TPopupMenuGroup.setIconSelectedForecolor(f_iconSelectedForeColor: TColor);
var
  l_i: integer;
begin
  m_iconSelectedForeColor := f_iconSelectedForeColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].IconSelectedForeColor := f_iconSelectedForeColor;
end;


procedure TPopupMenuGroup.SetIcon3dRightBottomEdgeBordersColor(f_icon3dRightBottomEdgeBordersColor: TColor);
var
  l_i: integer;
begin
  m_icon3dRightBottomEdgeBordersColor := f_icon3dRightBottomEdgeBordersColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].Icon3dRightBottomEdgeBordersColor := f_icon3dRightBottomEdgeBordersColor;
end;

procedure TPopupMenuGroup.SetIcon3dLeftTopEdgeBordersColor(f_icon3dLeftTopEdgeBordersColor: TColor);
var
  l_i: integer;
begin
  m_icon3dLeftTopEdgeBordersColor := f_icon3dLeftTopEdgeBordersColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].Icon3dLeftTopEdgeBordersColor := f_icon3dLeftTopEdgeBordersColor;
end;

procedure TPopupMenuGroup.SetIcon3dRightSelectedBottomEdgeBordersColor(f_icon3dRightSelectedBottomEdgeBordersColor: TColor);
var
  l_i: integer;
begin
  m_icon3dRightSelectedBottomEdgeBordersColor := f_icon3dRightSelectedBottomEdgeBordersColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].Icon3dRightSelectedBottomEdgeBordersColor := f_icon3dRightSelectedBottomEdgeBordersColor;
end;


procedure TPopupMenuGroup.SetIcon3dLeftSelectedTopEdgeBordersColor(f_icon3dLeftSelectedTopEdgeBordersColor: TColor);
var
  l_i: integer;
begin
  m_icon3dLeftSelectedTopEdgeBordersColor := f_icon3dLeftSelectedTopEdgeBordersColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].Icon3dLeftSelectedTopEdgeBordersColor := f_icon3dLeftSelectedTopEdgeBordersColor;
end;

procedure TPopupMenuGroup.setDisabledColor(f_disabledColor: TColor);
var
  l_i: integer;
begin
  m_disabledColor := f_disabledColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].DisabledColor := f_disabledColor;
end;

procedure TPopupMenuGroup.setDisabledSelectedColor(f_disabledSelectedColor: TColor);
var
  l_i: integer;
begin
  m_disabledSelectedColor := f_disabledSelectedColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].DisabledSelectedColor := f_disabledSelectedColor;
end;

procedure TPopupMenuGroup.setBorderColor(f_BorderColor: TColor);
var
  l_i: integer;
begin
  m_borderColor := f_BorderColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].BorderColor := f_BorderColor;
end;

procedure TPopupMenuGroup.setFontfilePath(f_path: string);
var
  l_i: integer;
begin
  m_fontfilepath := f_path;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].setFontfilePath(f_path);
end;


procedure TPopupMenuGroup.setFont(f_fontFile: string; f_fontSize: integer; f_fontstyles: string);
var
  l_i: integer;
begin
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].setFont(f_fontFile,f_fontSize,f_fontstyles);
end;

procedure TPopupMenuGroup.setBlankColor(f_blankColor: TColor);
var
  l_i: integer;
begin
  m_BlankColor := f_blankColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].BlankColor := f_blankColor;
end;

procedure TPopupMenuGroup.setBackColor(f_backColor: TColor);
var
  l_i: integer;
begin
  m_Backcolor := f_Backcolor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].Backcolor := f_Backcolor;
end;

procedure TPopupMenuGroup.setSelectedBackColor(f_SelectedBackcolor: TColor);
var
  l_i: integer;
begin
  m_SelectedBackcolor := f_SelectedBackcolor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].SelectedBackColor := f_SelectedBackcolor;
end;

procedure TPopupMenuGroup.setFontColor(f_fontColor: TColor);
var
  l_i: integer;
begin
  m_fontColor := f_Fontcolor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].FontColor := f_Fontcolor;
end;

procedure TPopupMenuGroup.setSelectedFontColor(f_selectedFontColor: TColor);
var
  l_i: integer;
begin
  m_selectedFontColor := f_selectedFontColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].SelectedFontColor := f_selectedFontColor;
end;

procedure TPopupMenuGroup.setCheckedColorColor(f_checkedColor: TColor);
var
  l_i: integer;
begin
  m_checkedColor := f_checkedColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].CheckedColor := f_checkedColor;
end;

procedure TPopupMenuGroup.setSelectedCheckedColor(f_selectedCheckedColor: TColor);
var
  l_i: integer;
begin
  m_selectedCheckedColor := f_selectedCheckedColor;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].SelectedCheckedColor := f_selectedCheckedColor;
end;

procedure TPopupMenuGroup.setTextLeft(f_textLeft: integer);
var
  l_i: integer;
begin
  m_textLeft := f_textLeft;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].TextLeft := f_textLeft;
end;

procedure TPopupMenuGroup.setTextRight(f_textRight: integer);
var
  l_i: integer;
begin
  m_textRight := f_TextRight;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].TextRight := f_textRight;
end;

procedure TPopupMenuGroup.setBlankHeight(f_blankHeight: integer);
var
  l_i: integer;
begin
  m_blankHeight := f_blankHeight;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].BlankHeigt := f_blankHeight;
end;

procedure TPopupMenuGroup.setItemsHeight(f_itemsHeight: integer);
var
  l_i: integer;
begin
  m_itemsheight := f_itemsHeight;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].ItemsHeight := f_itemsHeight;
end;

procedure TPopupMenuGroup.setItemsWidth(f_itemsWidth: integer);
var
  l_i: integer;
begin
  m_itemswidth := f_itemsWidth;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].ItemsWidth := f_itemsWidth;
end;

procedure TPopupMenuGroup.setRelativWidth(f_relativWidth: boolean);
var
  l_i: integer;
begin
  m_relativWidth := f_relativWidth;
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].RelativWidth := f_relativWidth;
end;

destructor TPopupMenuGroup.Destroy;
var
  l_i: integer;
begin
  for l_i := 0 to high(m_popupmenus) do
        m_popupmenus[l_i].Destroy;
  inherited Destroy;
end;

constructor TPopupMenuGroup.Create(AOwner: TComponent);
begin
  m_languageCommands := TStringList.Create;
  m_rootsOfDynamicSubmenues := TStringlist.Create;
  m_uncheckedItemsOnPopup := TStringList.Create;
  inherited Create(AOwner);
end;

function TPopupMenuGroup.IsLanguageFileCommand(f_ItemName: string): boolean;
var
  l_i: integer;
begin
  Result := false;
  if not (m_language = nil) then begin
        If f_ItemName = m_language.getActiveLanguageAsDescription then begin
                g_errLog.AddErrStr('TPopupMenuGroup.IsLanguageFileCommand: Language "' + f_ItemName + '" is already active');
                exit;
        end;
        for l_i := 0 to m_languageCommands.Count-1 do begin
                if f_ItemName = m_languageCommands[l_i] then begin
                        Result := true;
                        break;
                end;
        end;
  end;
end;

function TPopupMenuGroup.SetPopupmenusLanguageCaptions(f_languageDescription: string): boolean;
var
  l_captions: TStrings;
  l_i: Integer;
begin
  Result := true;
  l_captions := TStringList.Create;
  // set the languageFile
  if not m_language.setLanguageFileByDecription(f_languageDescription) then begin
        g_errLog.AddErrStr('TPopupMenuGroup.SetPopupmenusLanguageCaptions: SetLanguageFile by Description failed');
        Result := false;
        exit;
  end;
  for l_i := 0 to high(m_popupmenus) do
        // get and set all popupmenus ItemCaptions
        if not m_language.getPopupMenuCaptions(l_captions,m_popupmenus[l_i].getDefFileMenuItemNames) then begin
                g_errLog.AddErrStr('TPopupMenuGroup.SetPopupmenusCaptions: get Menucaptions failed');
                Result := false;
        end else
                m_popupmenus[l_i].setMenuEntryCaptions(l_captions);
  l_captions.free;
end;

procedure TPopupMenuGroup.OnDspPopupMenuItemClick(Sender: TObject; f_ItemName: string);
begin
  //g_errLog.AddErrStr('TPopupMenuGroup.OnDspPopupMenuItemClick: ' + f_ItemName);
  if IsLanguageFileCommand(f_ItemName) then begin
        if not SetPopupmenusLanguageCaptions(f_ItemName) then
                g_errLog.AddErrStr('TPopupMenuGroup.OnMenuItemClick: Change the Language Failed')
        else
                g_errLog.AddErrStr('TPopupMenuGroup.OnMenuItemClick: Changed the Language to "' + f_ItemName + '" successfully');
  end;
  if assigned(m_onMenuItemClick) then
        m_onMenuItemClick(Sender,f_ItemName);
end;

// SetupPopupMenu needs to be configured on every popup
procedure TPopupMenuGroup.OnMenuClientMenuPopup(Sender: TObject);
var
  l_i: integer;
  l_j: integer;
begin
  //g_errLog.AddErrStr('TPopupMenuGroup.OnMenuPopup');
  // Remove all dynamic submenues
  for l_j := 0 to m_rootsOfDynamicSubmenues.Count-1 do
        if TdspPopupmenu(Sender).getItemByName(m_rootsOfDynamicSubmenues[l_j]) <> nil then begin
                TdspPopupmenu(Sender).RemoveSubMenu(m_rootsOfDynamicSubmenues[l_j]);
                if assigned(m_onSubmenuDelete) then
                        m_onSubmenuDelete(Sender,m_rootsOfDynamicSubmenues[l_j]);
        end;
  // Remove all dynamic submenues
  for l_j := 0 to m_uncheckedItemsOnPopup.Count-1 do
        if TdspPopupmenu(Sender).getItemByName(m_uncheckedItemsOnPopup[l_j]) <> nil then begin
                TdspPopupmenu(Sender).setChecked(false, m_uncheckedItemsOnPopup[l_j]);
                if assigned(m_onUncheckItem) then
                        m_onUncheckItem(Sender,m_uncheckedItemsOnPopup[l_j]);
        end;
  if not (m_language = nil) then begin
        if TdspPopupmenu(Sender).getItemByName(gc_language) <> nil then begin
                TdspPopupmenu(Sender).RemoveSubMenu(gc_language);
                m_language.getLanguageDescriptions(m_languageCommands);
                for l_i := 0 to m_languageCommands.Count-1 do begin
                        TdspPopupmenu(Sender).addItem(m_languageCommands[l_i],m_languageCommands[l_i],gc_language);
                        TdspPopupmenu(Sender).setRadio(true,m_languageCommands[l_i]);
                end;
                // Check the active LanguageItem
                TdspPopupmenu(Sender).setChecked(true,m_language.getActiveLanguageAsDescription);
        end;
  end;
  // send the event to Receiver if assigned
 if assigned(m_onMenuPopup) then
        m_onMenuPopup(Sender);
end;



end.

